@@ -31,102 +31,6 @@ open CommonTypes
 open CommonFile
 
 
-(*************************************************************************)
-(*                                                                       *)
-(*                         FILES                                         *)
-(*                                                                       *)
-(*************************************************************************)
-
-module FileOption = struct
-    
-    let value_to_state v =
-      match v with
-      | StringValue "Paused" -> FilePaused
-      | StringValue "Downloading" -> FileDownloading
-      | StringValue "Downloaded" -> FileDownloaded
-      | _ -> raise Not_found
-    
-    let state_to_value s = 
-      match s with
-      | FilePaused | FileAborted _ -> StringValue "Paused"
-      | FileDownloaded -> StringValue "Downloaded"
-      | _ -> StringValue "Downloading"
-    
-    
-    let value_to_file is_done v =
-      match v with
-        Options.Module assocs ->
-          let get_value name conv = conv (List.assoc name assocs) in
-          let network = try get_value "file_network" value_to_string
-            with _ -> "Donkey" in
-          let network = network_find_by_name network in
-          let file_state = 
-            try
-              get_value "file_state" value_to_state 
-            with _ -> FileDownloading
-          in          
-          let file_size = try
-              value_to_int64 (List.assoc "file_size" assocs) 
-            with _ -> Int64.zero
-          in
-          
-          let file = network_file_of_option network file_size 
-              file_state assocs in
-          let priority = try get_value "file_priority" value_to_int 
-            with _ -> 0 in
-          
-          let impl = as_file_impl file in
-          (try
-              impl.impl_file_age <- 
-                normalize_time (get_value "file_age" value_to_int)
-            with _ -> ());
-          set_file_state file file_state;       
-          (match file_state with
-              FileDownloading -> lprintf "New downloading file\n";
-            | FileDownloaded -> lprintf "New downloaded file\n";
-            | _ -> lprintf "..........\n"
-          );
-          
-          (try
-              set_file_best_name file
-              (get_value "file_filename" value_to_string)
-            with _ -> ());
-          set_file_priority file priority;
-          file
-      | _ -> assert false
-          
-    let file_to_value file =
-        let netname = string_to_value (file_network file).network_name in
-      let impl = as_file_impl file in
-      Options.Module (
-        ("file_network", netname) ::
-        ("file_size", int64_to_value (file_size file)) ::
-        ("file_priority", int_to_value (file_priority file)) ::
-        ("file_state", state_to_value (file_state file)) ::
-        ("file_filename", string_to_value (file_best_name file)) ::
-        ("file_age", IntValue (Int64.of_int impl.impl_file_age)) ::
-          (file_to_option file)
-        )
-          
-    let t is_done =
-      define_option_class "File" (value_to_file is_done) file_to_value
-    ;;
-  end
-
-
-let swarmers_section = file_section files_ini [] ""
-let files_section = file_section files_ini [] "" 
-  
-let done_files = 
-  define_option files_section ["done_files"] 
-    "The files whose download is finished" (
-    listiter_option (FileOption.t true)) []
-  
-let files = 
-  define_option files_section ["files"] 
-    "The files currently being downloaded" (
-    listiter_option (FileOption.t false)) []
-
 (*************************************************************************)
 (*                                                                       *)
 (*                         RESULTS                                       *)
@@ -680,12 +584,16 @@ module SharedDirectoryOption = struct
             in
             let shdir_strategy = get_value_safe "strategy"
                 value_to_string "only_directory"
-            in
+	    in
+	    let shdir_upload_lifetime = get_value_safe "upload_lifetime" value_to_int 90 in
+	    let shdir_chunks_percent = get_value_safe "chunks_percent" value_to_int 100 in
             {
               shdir_dirname = shdir_dirname; 
               shdir_strategy = shdir_strategy; 
               shdir_networks = shdir_networks; 
               shdir_priority = shdir_priority; 
+	      shdir_upload_lifetime = shdir_upload_lifetime;
+	      shdir_chunks_percent = 100;
             }
           end
       
@@ -695,7 +603,9 @@ module SharedDirectoryOption = struct
             shdir_dirname = value_to_filename dir; 
             shdir_strategy = "only_directory"; 
             shdir_networks = []; 
-            shdir_priority = value_to_int prio; 
+            shdir_priority = value_to_int prio;
+	    shdir_upload_lifetime = 90;
+	    shdir_chunks_percent = 100;
           }          
       | v -> 
           let dir = value_to_string v in
@@ -704,7 +614,9 @@ module SharedDirectoryOption = struct
             shdir_dirname = dir; 
             shdir_strategy = "only_directory"; 
             shdir_networks = []; 
-            shdir_priority = prio; 
+            shdir_priority = prio;
+	    shdir_upload_lifetime = 90;
+	    shdir_chunks_percent = 100;
           }
           
     let shared_directory_to_value s =
@@ -715,6 +627,8 @@ module SharedDirectoryOption = struct
           "strategy", 
           string_to_value  s.shdir_strategy;
           "priority", int_to_value s.shdir_priority;
+	  "upload_lifetime", int_to_value s.shdir_upload_lifetime;
+	  "chunks_percent", int_to_value s.shdir_chunks_percent;
         ]
       in
       Options.Module list
@@ -732,15 +646,151 @@ let shared_directories =
     (list_option SharedDirectoryOption.t) 
   [
     {
-      shdir_dirname = "shared";
+      shdir_dirname = (Filename.concat file_basedir "incoming");
       shdir_priority = 0;
       shdir_networks = [];
       shdir_strategy = "all_files";
+      shdir_upload_lifetime = 90;
+      shdir_chunks_percent = 100;
     }
   ]
 
-
+module SharedDirectoryIndexOption = struct
+    let index_to_shared_directory v =
+      let i = value_to_int v in
+      try 
+        List.nth !!shared_directories i
+      with Failure "nth" ->
+      try
+        List.hd !!shared_directories
+      with Failure "hd" ->
+        restore_default shared_directories;
+	List.hd !!shared_directories
     
+    let shared_directory_to_index sh =
+      let res = ref 0 in
+      ignore (List.fold_left (fun i s ->
+        if s.shdir_dirname = sh.shdir_dirname then
+	  res := i;
+	i + 1
+      ) 0 !!shared_directories);
+      int_to_value !res
+      
+     let t =
+      define_option_class "Shared Directory Index" index_to_shared_directory
+        shared_directory_to_index
+   
+  end
+  
+let default_incoming = define_option current_section ["default_incoming" ]
+  "Shared directory where files will be commited per default"
+  SharedDirectoryIndexOption.t
+  (List.hd !!shared_directories)
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         FILES                                         *)
+(*                                                                       *)
+(*************************************************************************)
+
+module FileOption = struct
+
+    open SharedDirectoryIndexOption
+    
+    let value_to_state v =
+      match v with
+      | StringValue "Paused" -> FilePaused
+      | StringValue "Downloading" -> FileDownloading
+      | StringValue "Downloaded" -> FileDownloaded
+      | _ -> raise Not_found
+    
+    let state_to_value s = 
+      match s with
+      | FilePaused | FileAborted _ -> StringValue "Paused"
+      | FileDownloaded -> StringValue "Downloaded"
+      | _ -> StringValue "Downloading"
+    
+      
+    let value_to_file is_done v =
+      match v with
+        Options.Module assocs ->
+          let get_value name conv = conv (List.assoc name assocs) in
+          let network = try get_value "file_network" value_to_string
+            with _ -> "Donkey" in
+          let network = network_find_by_name network in
+          let file_state = 
+            try
+              get_value "file_state" value_to_state 
+            with _ -> FileDownloading
+          in          
+          let file_size = try
+              value_to_int64 (List.assoc "file_size" assocs) 
+            with _ -> Int64.zero
+          in
+          
+          let file = network_file_of_option network file_size 
+              file_state assocs in
+          let priority = try get_value "file_priority" value_to_int 
+            with _ -> 0 in
+          
+          let impl = as_file_impl file in
+          (try
+              impl.impl_file_age <- 
+                normalize_time (get_value "file_age" value_to_int)
+            with _ -> ());
+          set_file_state file file_state;       
+          (match file_state with
+              FileDownloading -> lprintf "New downloading file\n";
+            | FileDownloaded -> lprintf "New downloaded file\n";
+            | _ -> lprintf "..........\n"
+          );
+          
+          (try
+              set_file_best_name file
+              (get_value "file_filename" value_to_string)
+            with _ -> ());
+          set_file_priority file priority;
+	  (try 
+	      set_file_dst_share file (Some 
+	        (get_value "file_dst_share" index_to_shared_directory))
+	   with _ -> set_file_dst_share file None);
+          file
+      | _ -> assert false
+          
+    let file_to_value file =
+        let netname = string_to_value (file_network file).network_name in
+      let impl = as_file_impl file in
+      Options.Module (
+        ("file_network", netname) ::
+        ("file_size", int64_to_value (file_size file)) ::
+        ("file_priority", int_to_value (file_priority file)) ::
+        ("file_state", state_to_value (file_state file)) ::
+        ("file_filename", string_to_value (file_best_name file)) ::
+        ("file_age", IntValue (Int64.of_int impl.impl_file_age)) ::
+	(match file_dst_share file with
+	  Some sh -> [ ("file_dst_share", shared_directory_to_index sh) ]
+	  | None  -> []) @
+        (file_to_option file)
+      )
+          
+    let t is_done =
+      define_option_class "File" (value_to_file is_done) file_to_value
+    ;;
+  end
+
+let files_section = file_section files_ini [] "" 
+let swarmers_section = file_section files_ini [] ""
+
+let done_files = 
+  define_option files_section ["done_files"] 
+    "The files whose download is finished" (
+    listiter_option (FileOption.t true)) []
+  
+let files = 
+  define_option files_section ["files"] 
+    "The files currently being downloaded" (
+    listiter_option (FileOption.t false)) []
+
 (*************************************************************************)
 (*                                                                       *)
 (*                         Functions                                     *)
