@@ -49,6 +49,7 @@ type 'a file_impl = {
     mutable impl_file_best_name : string;
     mutable impl_file_priority: int; (* normal = 0, low < 0, high > 0 *)
     mutable impl_file_last_seen : int;
+    mutable impl_file_dst_share : CommonTypes.shared_directory option;
   }
 
 
@@ -75,10 +76,10 @@ it will happen soon. *)
     mutable op_file_active_sources : ('a -> client list);
     mutable op_file_comment : ('a -> string);
     mutable op_file_set_priority : ('a -> int -> unit);
+    mutable op_file_print_html : ('a -> Buffer.t -> unit);
     mutable op_file_print_sources_html : ('a -> Buffer.t -> unit);
     mutable op_file_files : ('a -> 'a file_impl -> file list);    
 (* added in 2.5.27 to remove use of network names in global modules *)
-    mutable op_file_print_sources_html_header : ('a -> Buffer.t -> GuiTypes.file_info -> unit);
     mutable op_file_debug : ('a -> string);
   }
 
@@ -117,6 +118,7 @@ let dummy_file_impl = {
     impl_file_priority = 0;
     impl_file_last_seen = 0;
     impl_file_comment = "";
+    impl_file_dst_share = None;
   }
   
 let dummy_file = as_file dummy_file_impl  
@@ -234,6 +236,14 @@ let file_comment file =
   let impl = as_file_impl file in
   impl.impl_file_comment
       
+let file_dst_share file =
+  let file = as_file_impl file in
+  file.impl_file_dst_share
+
+let set_file_dst_share file share =
+  let file = as_file_impl file in
+  file.impl_file_dst_share <- share
+
 let file_best_name (file : file) =
   let file = as_file_impl file in
   file.impl_file_best_name
@@ -273,11 +283,56 @@ let file_active_sources file =
   let impl = as_file_impl file in
   try impl.impl_file_ops.op_file_active_sources impl.impl_file_val with _ -> []
 
+(* Default for networks that don't implement it *)
+let default_file_print_sources_html file buf = 
+  let cfile = as_file file in
+  let allsources = ref (file_all_sources cfile) in
+  if List.length !allsources > 0 then begin
+
+    html_mods_table_header buf "sourcesTable" "sources al" [ 
+        ( "1", "srh br ac", "Client number", "Num" ) ; 
+        ( "0", "srh br", "Client Name", "Name" ) ; 
+        ( "0", "srh br", "IP address", "IP address" ) ; 
+        ( "1", "srh ar", "Total UL bytes to this client for all files", "UL" ) ; 
+        ( "1", "srh ar br", "Total DL bytes from this client for all files", "DL" ) ; ];
+
+    html_mods_cntr_init ();
+
+    List.iter (fun c ->
+        let cinfo = client_info c in
+        let addr = 
+        (match cinfo.GuiTypes.client_kind with
+          Indirect_location (name, _) -> ""
+        | Known_location (ip, port) -> Printf.sprintf "%s:%d" (Ip.to_string ip) port 
+        )
+        in
+
+        Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr());
+
+        html_mods_td buf [
+          ("", "sr br ar", Printf.sprintf "%d" (client_num c));
+          ("", "sr br", cinfo.GuiTypes.client_name);
+          ("", "sr br", addr);
+          ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_uploaded));
+          ("", "sr ar br", (size_of_int64 cinfo.GuiTypes.client_downloaded)); ];
+
+        Printf.bprintf buf "\\</tr\\>";
+
+    ) !allsources;
+
+    Printf.bprintf buf "\\</table\\>\\</div\\>\\<br\\>";
+
+  end
+
+
 let file_print_sources_html (file : file) buf =
   let file = as_file_impl file in
-  file.impl_file_ops.op_file_print_sources_html file.impl_file_val buf
-
+  try file.impl_file_ops.op_file_print_sources_html file.impl_file_val buf with _ -> 
+    default_file_print_sources_html file buf
   
+let file_print_html file buf =
+  let impl = as_file_impl file in
+  impl.impl_file_ops.op_file_print_html impl.impl_file_val buf
   
   
 let file_find num = 
@@ -480,135 +535,58 @@ let file_print file o =
         ("", "sr", Printf.sprintf "%d" (List.length srcs)) ];
       
       Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-1\\\"\\>";
+      let tt = ref "0=Missing, 1=Partial, 2=Complete, 3=Verified" in
       html_mods_td buf [
-        ("0=Missing, 1=Partial, 2=Complete, 3=Verified", "sr br", "Chunks");
-        ("0=Missing, 1=Partial, 2=Complete, 3=Verified", "sr", info.G.file_chunks) ];
-      
-      (match n.network_name with 
-          "BitTorrent" -> (
-              Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-2\\\"\\>";
-              html_mods_td buf [
-                ("Filename", "sr br", "Filename");
-                ("", "sr", (file_best_name file)) ]; )
-        
-        | _ -> (
-              
-              Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-2\\\"\\>";
-              html_mods_td buf [
-                ("Fake check links", "sr br", "Fakecheck");
-                ("", "sr", Printf.sprintf "\\<a target=\\\"_blank\\\" href=\\\"http://donkeyfakes.gambri.net/fakecheck.php\\?hash=%s\\\"\\>[Donkey-Fakes]\\</a\\>" 
-                    (Md4.to_string info.G.file_md4) 
-                ) ];
-              
-              
-              Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-1\\\"\\>";
-              
-              html_mods_td buf [
-                ("ed2k link", "sr br", "ed2k link");
-                ("", "sr", Printf.sprintf "\\<a href=\\\"ed2k://|file|%s|%s|%s|/\\\"\\>ed2k://|file|%s|%s|%s|/\\</A\\>" 
-                    (info.G.file_name)
-                  (Int64.to_string info.G.file_size)
-                  (Md4.to_string info.G.file_md4)
-                  (info.G.file_name)
-                  (Int64.to_string info.G.file_size)
-                  (Md4.to_string info.G.file_md4)) ];
-              
-              Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-2\\\"\\>";
-              let optionlist = ref "" in
-              List.iter (fun (name,_) -> 
-                  optionlist := !optionlist ^ Printf.sprintf "\\<option value=\\\"%s\\\"\\>%s\n" name name;
-              ) info.G.file_names;
-              
-              
-              let input_size = (Printf.sprintf "%d" ((String.length (file_best_name file))+3)) in
-              html_mods_td buf [
-                ("Rename file by selecting an alternate name", "sr br", "Filename");
-                ("Rename file", "sr", 
-                  Printf.sprintf "\\<script language=javascript\\>
-\\<!-- 
-function submitRenameForm(i) {
-var formID = document.getElementById(\\\"renameForm\\\" + i)
-var regExp = new RegExp (' ', 'gi') ;
-var renameTextOut = formID.newName.value.replace(regExp, '+'); 
-parent.fstatus.location.href='submit?q=rename+%d+\\\"'+renameTextOut+'\\\"';
-}
-//--\\>
-\\</script\\>" (file_num file) 
+        (!tt, "sr br", "Chunks");
+        (!tt, "sr", info.G.file_chunks) ];
                   
-                  ^ "\\<table border=0 cellspacing=0 cellpadding=0\\>\\<tr\\>\\<td\\>"
-                    ^ "\\<form name=\\\"renameForm1\\\" id=\\\"renameForm1\\\" action=\\\"javascript:submitRenameForm(1);\\\"\\>"
-                    ^ "\\<select name=\\\"newName\\\" id=\\\"newName\\\" onchange=\\\"this.form.submit()\\\"\\>"
-                    ^ Printf.sprintf "\\<option value=\\\"%s\\\" selected\\>%s\n" (file_best_name file) (file_best_name file)
-                  ^ !optionlist 
-                    ^ "\\</select\\>\\</td\\>\\</tr\\>\\</form\\>\\<tr\\>\\<td\\>\\<form name=\\\"renameForm2\\\" id=\\\"renameForm2\\\" action=\\\"javascript:submitRenameForm(2);\\\"\\>"
-                    ^ "\\<input name=\\\"newName\\\" type=text size=" ^ input_size ^ " value=\\\"" ^ (file_best_name file) ^ "\\\"\\>\\</input\\>\\</td\\>\\</tr\\>\\</form\\>\\</table\\>"
                 
-                ) ];
-            
-            )
-      );
+      file_print_html file buf;
       
       Printf.bprintf buf "\\</tr\\>\\</table\\>\\</div\\>"; 
       Printf.bprintf buf "\\</td\\>\\</tr\\>\\</table\\>\\</div\\>\\<br\\>";
     
     end else
     begin
-      Printf.bprintf buf "[%-s %5d] %s %10s %10s\npriority %d\n" 
+      Printf.bprintf buf "[%-s %5d] %s %10s %10s\npriority %d dst-share %s\n" 
         n.network_name (file_num file) 
       (match info.G.file_names with
           [] -> Md4.to_string info.G.file_md4
         | (name,_) :: _ -> String2.shorten !!max_name_len name)
       (Int64.to_string info.G.file_size)
       (Int64.to_string info.G.file_downloaded)
-      (file_priority file);
+      (file_priority file)
+      (match file_dst_share file with
+        Some sh -> sh.shdir_dirname
+	| None	-> "<default>");
       Printf.bprintf buf "Chunks: [%-s]\n" info.G.file_chunks;
       List.iter (fun (name,_) -> Printf.bprintf buf "    (%s)\n" name) info.G.file_names
     end;
   
   (try
             
-      
       if !!print_all_sources then begin
-          
-          if not (use_html_mods o) then 
+          if use_html_mods o then
+            file_print_sources_html file buf
+          else begin
             Printf.bprintf buf "%d sources:\n" (List.length srcs);
-          
-          if use_html_mods o && srcs <> [] then begin
-
-(* [TODO] Hum... we should add a method in network for that, not use the name
-of the network !!! *)
-              impl.impl_file_ops.op_file_print_sources_html_header 
-                file buf info;
-            end;
-          
-          
           let counter = ref 0 in
           let print_source c =
-              incr counter;
-              
-              if use_html_mods o then begin
-                  Printf.bprintf buf " \\<tr onMouseOver=\\\"mOvr(this);\\\" 
-				onMouseOut=\\\"mOut(this);\\\" class=\\\"%s\\\"\\>"
-                    (if (!counter mod 2 == 0) then "dl-1" else "dl-2");
-                  client_bprint_html c buf file;
-                  Printf.bprintf buf "\\</tr\\>";
-                end
-              else begin
                   Printf.bprintf buf "  [%4d] " (client_num c);
                   client_bprint c buf;
-              end
           in
           List.iter print_source srcs;
-        
         end;
-      if use_html_mods o && srcs <> [] then begin
-          Printf.bprintf buf "\\</table\\>\\</div\\>\\<br\\>";
-          if !!html_mods_vd_queues then file_print_sources_html file buf;
         end
     
     with _ -> ())
   
 
+
+
+
+  
+
 (*************************************************************************)
 (*                                                                       *)
 (*                         add_segment (internal)                        *)
@@ -755,9 +733,9 @@ let new_file_ops network =
       op_file_active_sources = (fun _ -> fni network "file_active_sources");
       op_file_comment = (fun _ -> ni_ok network "file_comment"; "");
       op_file_set_priority = (fun _ _ -> ni_ok network "file_set_priority");
-      op_file_print_sources_html = (fun _ _ -> ni_ok network "file_print_sources_html");
+      op_file_print_html = (fun _ _ -> ni_ok network "file_print_html");
+      op_file_print_sources_html = (fun _ _ -> fni network "file_print_sources_html");
       op_file_debug = (fun _ -> "");
-      op_file_print_sources_html_header = (fun _ _ _ -> ());
     }
   in
   let ff = (Obj.magic f : int file_ops) in
@@ -796,6 +774,8 @@ let check_file_implementations () =
         lprintf "op_file_all_sources\n";
       if c.op_file_active_sources == cc.op_file_active_sources then
         lprintf "op_file_active_sources\n";
+      if c.op_file_print_html == cc.op_file_print_html then
+        lprintf "op_file_print_html\n";
       if c.op_file_print_sources_html == cc.op_file_print_sources_html then
         lprintf "op_file_print_sources_html\n";
   ) !files_ops;
@@ -838,6 +818,7 @@ let file_verify file key begin_pos end_pos =
       lprintf "Checksum to compute: %Ld-%Ld of %s\n" begin_pos end_pos
       (file_disk_name file);
     end;
+ try
   let computed = match key with
   | Ed2k md4 ->
       let result = Md4.digest_subfile (file_fd file) 
@@ -858,6 +839,11 @@ let file_verify file key begin_pos end_pos =
        (if result then "VERIFIED" else "CORRUPTED");
     end;
   result
+ with
+ | Not_found-> raise Not_found
+ | _ -> if !verbose_md4 then
+ lprintf "Checksum computed: chunk MISSING\n";
+ false 
 
 let file_mtime file = Unix32.mtime64 (file_fd file)
   
