@@ -27,6 +27,12 @@ open UdpSocket
 open TcpBufferedSocket
 
 
+(* This one is networkspecific, however it has to be here, since commonServer will use it! *)
+let donkeyIsLowID ip =
+  match Ip.to_ints ip with
+    | _, _, _, 0 -> true
+    | _ -> false
+
 (*************************************************************************)
 (*                                                                       *)
 (*                         short_lazy                                    *)
@@ -71,10 +77,14 @@ module ShortLazy : sig
 
 let networks_string = ref ""
   
+let patches_string = ref "No patches were used."
+
 let version () = 
   Printf.sprintf "MLNet %s: Multi-Network p2p client (%s)"  
     Autoconf.current_version !networks_string
   
+let buildinfo () =
+  Printf.sprintf "MLNet %s: Multi-Network p2p client (%s)\nwas built with %s\n%s" Autoconf.current_version !networks_string Autoconf.build_system !patches_string
   
 (* Should we try to find another port when we cannot bind to the one set
 in an option, and then change the option accordingly. ?> *)
@@ -464,6 +474,13 @@ let chat_message_fifo = (Fifo.create () : (int * string * int * string * string)
 
 let log_chat_message i num n s = 
   Fifo.put chat_message_fifo (last_time(),i,num,n,s);
+  try
+    let oc = open_out_gen [Open_creat; Open_wronly; Open_append] 0o600 !messages_log in
+    Printf.fprintf oc "%s: %s (%s): %s\n" (Date.simple (BasicSocket.date_of_int (last_time ()))) n i s;
+    close_out oc;
+  with e ->
+    lprintf "[ERROR] Exception %s while trying to log message to %s\n"
+      (Printexc2.to_string e) !messages_log;
   
   while (Fifo.length chat_message_fifo) > !!html_mods_max_messages  do
     let foo = Fifo.take chat_message_fifo in ()
@@ -485,6 +502,41 @@ let html_mods_td buf l =
      c (if t <> "" then "title=\\\"" ^ t ^ "\\\"" else "") d;
     ) l
   
+let html_mods_counter = ref true
+
+let html_mods_cntr () =
+    html_mods_counter := not !html_mods_counter;
+    if !html_mods_counter then 1 else 2
+
+let html_mods_cntr_init () =
+  html_mods_counter := true
+
+(* ripped from gui_misc *)
+
+let ko = 1024.0 
+let mo = ko *. ko 
+let go = mo *. ko 
+let tob = go *. ko 
+
+let size_of_int64 size =
+  if !!html_mods_human_readable then
+    let f = Int64.to_float size in
+  if f > tob then
+      Printf.sprintf "%.2fT" (f /. tob)
+  else
+     if f > go then
+      Printf.sprintf "%.2fG" (f /. go)
+     else
+      if f > mo then
+      Printf.sprintf "%.1fM" (f /. mo)
+      else
+     if f > ko then
+       Printf.sprintf "%.1fk" (f /. ko)
+     else
+       Int64.to_string size
+  else
+    Int64.to_string size
+  
 let debug_clients = ref Intset.empty
 
 let default_user = {
