@@ -19,6 +19,7 @@
 
 open AnyEndian
 open LittleEndian
+open Int64ops
 open Printf2
 open CommonOptions
 open BasicSocket  
@@ -67,10 +68,11 @@ let file_commited_name file =
   let best_name = file_best_name file in
   let file_name = file_disk_name file in
   let incoming_dir =
+    let share = CommonShared.share_to_commit file in
     if network.network_incoming_subdir () <> "" then
-      Filename.concat !!incoming_directory
+      Filename.concat share.shdir_dirname
         (network.network_incoming_subdir ())
-    else !!incoming_directory
+    else share.shdir_dirname
   in
   (try Unix2.safe_mkdir incoming_dir with _ -> ());
   let new_name = 
@@ -109,16 +111,14 @@ let file_commit file =
             set_file_disk_name file new_name;
             let best_name = file_best_name file in  
             Unix32.destroy (file_fd file);
-            lprintf "destroyed\n";
+            if !verbose_files then lprintf "commonInteractive.file_commit: destroyed\n";
             let impl = as_file_impl file in
             impl.impl_file_ops.op_file_commit impl.impl_file_val new_name;
-            
+	    let share = CommonShared.share_to_commit file in
             begin
               try
                 if not (Unix2.is_directory new_name) then 
-                  ignore (CommonShared.new_shared 
-                      !!incoming_directory !!incoming_directory_prio
-                      best_name new_name);
+		      CommonShared.new_shared share best_name new_name;
               with e ->
                   lprintf "Exception %s while trying to share commited file\n"
                     (Printexc2.to_string e);
@@ -128,7 +128,7 @@ let file_commit file =
             done_files =:= List2.removeq file !!done_files;
             files =:= List2.removeq file !!files;
             
-            lprintf "going to secondaries...\n";
+            if !verbose_files then lprintf "commonInteractive.file_commit: going to secondaries...\n";
             List.iter (fun file ->
 (* Commit the file first, and share it after... *)
                 try
@@ -174,17 +174,12 @@ let mail_for_completed_file file =
     let module M = Mailer in
     let line1 = "mldonkey has completed the download of:\r\n\r\n" in
 
-    let line2 = Printf.sprintf "\r\nFile: %s\r\nSize: %Ld bytes\r\n" 
+    let line2 = Printf.sprintf "\r\nFile: %s\r\nSize: %Ld bytes\r\nComment: %s\r\n" 
       (file_best_name file)
       (file_size file)
+      (file_comment file)
     in
     
-    let line3 = if (file_comment file) <> "" then
-	Printf.sprintf "\r\nComment: %s\r\n" (file_comment file)
-      else
-        Printf.sprintf "";
-    in
-
     let subject = if !!filename_in_subject then
         Printf.sprintf "[mldonkey] file received - %s"
         (file_best_name file)
@@ -192,8 +187,8 @@ let mail_for_completed_file file =
         Printf.sprintf "mldonkey, file received";        
     in
 
-    let line4 = if !!url_in_mail <> "" then
-	Printf.sprintf "\r\n%s/%s\r\n" !!url_in_mail (file_best_name file)
+    let line3 = if !!url_in_mail <> "" then
+	Printf.sprintf "\r\n<%s/%s>\r\n" !!url_in_mail (Url.encode (file_best_name file))
       else
         Printf.sprintf "";
     in
@@ -202,7 +197,7 @@ let mail_for_completed_file file =
         M.mail_to = !!mail;
         M.mail_from = !!mail;
         M.mail_subject = subject;
-        M.mail_body = line1 ^ line2 ^ line3 ^ line4;
+        M.mail_body = line1 ^ line2 ^ line3;
       } in
     M.sendmail !!smtp_server !!smtp_port !!add_mail_brackets mail
 
@@ -219,9 +214,9 @@ let file_completed (file : file) =
         update_file_state impl FileDownloaded;  
         let file_name = file_disk_name file in
         let file_id = Filename.basename file_name in
-        ignore (CommonShared.new_shared "completed" 0 (
+        (*ignore (CommonShared.new_shared "completed" 0 (
             file_best_name file )
-          file_name);
+          file_name);*)
         (try mail_for_completed_file file with e ->
               lprintf "Exception %s in sendmail\n" (Printexc2.to_string e);
               );
@@ -350,31 +345,6 @@ let time_of_sec sec =
 	else if minutes > 0 then Printf.sprintf "%d:%02d" minutes seconds
     	else Printf.sprintf "00:%02d" seconds
   
-(* ripped from gui_misc *)
-
-let ko = 1024.0 
-let mo = ko *. ko 
-let go = mo *. ko 
-let tob = go *. ko 
-
-let size_of_int64 size =
-  if !!html_mods_human_readable then
-    let f = Int64.to_float size in
-	if f > tob then
-      Printf.sprintf "%.2fT" (f /. tob)
-	else
-     if f > go then
-      Printf.sprintf "%.2fG" (f /. go)
-     else
-      if f > mo then
-      Printf.sprintf "%.1fM" (f /. mo)
-      else
-     if f > ko then
-       Printf.sprintf "%.1fk" (f /. ko)
-     else
-       Int64.to_string size
-  else
-    Int64.to_string size
 
   
 let display_vd = ref false
@@ -813,10 +783,12 @@ let force_download_quotas () =
   let files = List.sort (fun f1 f2 -> 
         let v = file_priority f2 - file_priority f1 in
         if v <> 0 then v else
-        let s1 = file_downloaded f1 in
-        let s2 = file_downloaded f2 in
-        if s1 = s2 then 0 else
-        if s2 > s1 then 1 else -1
+(* Try to download in priority files with fewer bytes missing
+   Rationale: once completed, it may allow to recover some disk space *)
+        let r1 = file_size f1 -- file_downloaded f1 in
+        let r2 = file_size f2 -- file_downloaded f2 in
+        if r1 = r2 then 0 else
+        if r2 < r1 then 1 else -1 
         )
     !!CommonComplexOptions.files in
   
