@@ -225,7 +225,7 @@ let new_network name flags prefix_option subdir_option =
       op_network_client_of_option =  (fun _ -> fni name "op_network_client_of_option");
       op_network_recover_temp = (fun _ -> ni_ok name "recover_temp");
       op_network_search = (fun _ _ -> ni_ok name "search");
-      op_network_share = (fun _ _ _ -> ni_ok name "share");
+      op_network_share = (fun _ _ _ _ -> ni_ok name "share");
       op_network_private_message = (fun _ _ -> ni_ok name "private message");
       op_network_connect_servers = (fun _ -> ni_ok name "connect_servers");
       op_network_forget_search = (fun _ -> ni_ok name "forget_search");
@@ -252,4 +252,4 @@ let new_network name flags prefix_option subdir_option =
     networks_string := Printf.sprintf "%s %s" !networks_string r.network_name;
   lprintf "Network %s registered\n" r.network_name;
   r
-  
\ No newline at end of file
+  
