@@ -125,6 +125,8 @@ let files_ini = create_options_file (
 let friends_ini = create_options_file (
     Filename.concat file_basedir "friends.ini")
 
+let messages_log = ref (Filename.concat file_basedir "messages.log")
+
 let servers_section = file_section servers_ini [] ""
   
 
@@ -251,7 +253,12 @@ let current_section = interfaces_section
 let allowed_ips = define_option current_section ["allowed_ips"]
     ~desc: "Allowed IPs"
   "list of IP address allowed to connect to the core via telnet/GUI/WEB
-list separated by spaces, wildcard=255 ie: use 192.168.0.255 for 192.168.0.* "
+list separated by spaces, wildcard=255 ie: use 192.168.0.255 for 192.168.0.* 
+! Put the IPs between quotes !
+EXAMPLE:   allowed_ips = [
+             '127.0.0.1';
+             '192.168.1.2'; 
+             '192.168.1.3';]"
     ip_list_option [Ip.localhost]  
         
 let gui_port = 
@@ -286,7 +293,7 @@ let telnet_bind_addr = define_expert_option current_section ["telnet_bind_addr"]
     Ip.option (Ip.of_inet_addr Unix.inet_addr_any)
 
 let print_all_sources =
-  define_expert_option current_section ["print_all_sources"] "Should *all* sources for a file be shown on HTML/telnet vd <num>" bool_option false
+  define_expert_option current_section ["print_all_sources"] "Should *all* sources for a file be shown on HTML/telnet vd <num>" bool_option true
     
 let improved_telnet =
   define_expert_option current_section ["improved_telnet"] "Improved telnet interface" bool_option true
@@ -478,6 +485,12 @@ let html_mods_vd_prio = define_expert_option current_section
 let html_mods_vd_queues = define_expert_option current_section
     ["html_mods_vd_queues"] "Whether to display the Queues in vd # output" bool_option true
 
+let html_mods_vd_queued = define_expert_option current_section
+    ["html_mods_vd_queued"] "Whether to display queued files in vd output" bool_option true
+
+let html_mods_vd_paused = define_expert_option current_section
+    ["html_mods_vd_paused"] "Whether to display paused files in vd output" bool_option true
+
 let html_vd_barheight = define_expert_option current_section
     ["html_vd_barheight"] "Change height of download indicator bar in vd output" int_option 2
 
@@ -601,6 +614,11 @@ let minimal_packet_size = define_expert_option current_section ["minimal_packet_
 available on the connection"
     int_option !TcpBufferedSocket.minimal_packet_size
 
+let socketKeepalive = define_expert_option current_section ["socketKeepalive"]
+  "Should a connection check if the peer we are connected to is still alive?
+  This implies some bandwidth-cost (with 200 connections ~10-20%)"
+    bool_option !BasicSocket.socketKeepalive
+
 let network_update_url = 
   define_expert_option current_section ["network_update_url"]
     "URL where mldonkey can download update information on the network"
@@ -797,16 +815,6 @@ let temp_directory = define_option current_section ["temp_directory" ]
     "The directory where temporary files should be put" 
     string_option (Filename.concat file_basedir "temp")
 
-let incoming_directory_prio =
-  define_expert_option current_section ["incoming_directory_prio" ]
-      "The upload prio of the incoming directory"
-          int_option 0
-
-let incoming_directory = 
-  define_option current_section ["incoming_directory" ] 
-    "The directory where downloaded files should be moved after commit" 
-    string_option (Filename.concat file_basedir "incoming")
-
 let default_sharing_strategy =
   define_option current_section ["default_sharing_strategy" ] 
     "Sharing strategy the share command will use when none is specified"
@@ -1204,23 +1212,32 @@ let verbosity = define_expert_option current_section ["verbosity"]
   "A space-separated list of keywords. Each keyword triggers
   printing information on the corresponding messages:
   mc : debug client messages
+  mr|raw : debug raw messages
   mct : debug emule clients tags
   ms : debug server messages
-  connect : debug connections
-  net : debug net
   verb : debug other
-  loc : debug source research
-  sp : debug source propagation 
   sm : debug source management
+  net : debug net
   do : some download warnings
   up : some upload warnings
   unk : unknown messages
   ov : overnet
+  loc : debug source research
   share: debug sharing
   md4 : md4 computation
+  connect : debug connections
   udp : udp messages
-  hc: http_client messages
-  hs: http_server messages
+  ultra|super : debug supernode
+  swarming : debug swarming
+  hc : http_client messages
+  hs : http_server messages
+  act : debug activity
+  bw : debug bandwidth
+  tor : debug .torrent loading
+  file : debug file handling
+  redir : debug redirector
+  unexp : debug unexpected messages
+  hid : print hidden errors messages
 "
     string_option ""
   
@@ -1374,11 +1391,12 @@ let _ =
       TcpBufferedSocket.max_buffer_size := maxi 50000 !!client_buffer_size
   )
 
-let verbose_msg_servers = ref false
 let verbose_msg_clients = ref false
+let verbose_msg_raw = ref false
 let verbose_msg_clienttags = ref false
-let verbose_sources = ref 0
+let verbose_msg_servers = ref false
 let verbose = ref false
+let verbose_sources = ref 0
 let verbose_download = ref false
 let verbose_upload = ref false
 let verbose_unknown_messages = ref false
@@ -1388,17 +1406,23 @@ let verbose_share = ref false
 let verbose_md4 = ref false
 let verbose_connect = ref false
 let verbose_udp = ref false
+let verbose_supernode = ref false
 let verbose_swarming = ref false
 let verbose_activity = ref false
+let verbose_torrent = ref false
+let verbose_files = ref false
+let verbose_redirector = ref false
+let verbose_unexpected_messages = ref false
+let verbose_hidden_errors = ref false
 
 let set_all v =
   
-  verbose_connect := v;
   verbose_msg_clients := v;
+  verbose_msg_raw := v;
   verbose_msg_clienttags := v;
   verbose_msg_servers := v;
-  BasicSocket.debug := v;
   verbose := v;
+  BasicSocket.debug := v;
   verbose_download := v;
   verbose_upload := v;
   verbose_unknown_messages := v;
@@ -1406,11 +1430,18 @@ let set_all v =
   verbose_location := v;
   verbose_share := v;
   verbose_md4 := v;
+  verbose_connect := v;
   verbose_udp := v;
+  verbose_supernode := v;
   verbose_swarming := v;
   Http_client.verbose := v;
   Http_server.verbose := v;
-  verbose_activity := v
+  verbose_activity := v;
+  verbose_torrent  := v;
+  verbose_files := v;
+  verbose_redirector := v;
+  verbose_unexpected_messages := v;
+  verbose_hidden_errors := v
   
 let _ = 
   option_hook verbosity (fun _ ->
@@ -1439,13 +1470,18 @@ let _ =
           | "hs" -> Http_server.verbose := true
           | "act" -> verbose_activity := true              
           | "bw" -> incr BasicSocket.verbose_bandwidth
+          | "tor" -> verbose_torrent := true
+          | "file" -> verbose_files := true
+          | "redir" -> verbose_redirector := true
+          | "unexp" -> verbose_unexpected_messages := true
+          | "hid" -> verbose_hidden_errors := true
               
           | "all" ->
               
               verbose_sources := 1;
               set_all true;
               
-          | _ -> ()
+          | _ -> lprintf "Unknown verbosity tag: %s" s
       
       ) (String2.split_simplify !!verbosity ' ')
   )
@@ -1454,6 +1490,9 @@ let _ =
 let _ =
   option_hook loop_delay (fun _ ->
      BasicSocket.loop_delay := (float_of_int !!loop_delay) /. 1000.;
+  );
+  option_hook socketKeepalive (fun _ ->
+      BasicSocket.socketKeepalive := !!socketKeepalive
   )
 
 let _ =
@@ -1510,4 +1549,4 @@ let update_options () =
       
       options_version =:= 1
       
-  | _ -> ()
\ No newline at end of file
+  | _ -> ()
