@@ -34,6 +34,7 @@ open CommonTypes
 type 'a shared_impl = {
     impl_shared_fullname : string;
     impl_shared_codedname : string;
+    impl_shared_directory : shared_directory;
     mutable impl_shared_val : 'a;
     mutable impl_shared_update : int;
     mutable impl_shared_num : int;
@@ -120,23 +121,21 @@ let shared_remove impl =
   H.remove shareds_by_num (as_shared impl)
     
 let dirnames = Hashtbl.create 13
-let dirnames_prio = ref []
 let dirname_counter = ref 0
 
 let files_scanned = ref 0
 let files_scanned_size = ref zero
   
-let new_shared dirname prio filename fullname =
+let new_shared shdir filename fullname =
 (*  lprintf "XXXXXXX\ndirname %s \nfilename %s \nfullname %s\n"
     dirname filename fullname;  *)
   let fullname = Filename2.normalize fullname in
   let filename = Filename2.normalize filename in
   let dirname = try
-      Hashtbl.find dirnames dirname with _ ->
+      Hashtbl.find dirnames shdir.shdir_dirname with _ ->
         incr dirname_counter;
         let name = Printf.sprintf "shared%d" !dirname_counter in
-        Hashtbl.add dirnames dirname name;
-	dirnames_prio := (name, prio) :: !dirnames_prio;
+        Hashtbl.add dirnames shdir.shdir_dirname name;
         name in
   let codedname = Filename.concat dirname filename in
   if !verbose_share then
@@ -151,7 +150,7 @@ let new_shared dirname prio filename fullname =
   end
   else begin
   CommonNetwork.networks_iter (fun n -> 
-      CommonNetwork.network_share n fullname codedname size);
+      CommonNetwork.network_share n shdir fullname codedname size);
   shared_calculate_total_bytes ();
   end
 
@@ -166,26 +165,15 @@ let shared_fullname s =
 let shared_codedname s =
   let impl = as_shared_impl s in
   impl.impl_shared_codedname  
+
+let shared_shared_directory s =
+  let impl = as_shared_impl s in
+  impl.impl_shared_directory
   
 let shared_unshare s =
   let impl = as_shared_impl s in
   try impl.impl_shared_ops.op_shared_unshare impl.impl_shared_val with _ -> ()
 
-let shared_dir = function
-  | None	-> ""
-  | Some sh	-> 
-    try
-      let impl = as_shared_impl sh in
-      let codedname = impl.impl_shared_codedname in
-      let idx = String.index codedname '/' in
-      String.sub codedname 0 idx
-    with Not_found -> ""
-		    
-let shared_prio impl =
-  try
-    List.assoc (shared_dir impl) !dirnames_prio
-  with Not_found -> 0
-
 let new_shared_ops network = {
     op_shared_unshare = (fun _ -> ni_ok network "shared_unshare");
     op_shared_info = (fun _ -> fni network "shared_info");
@@ -194,6 +182,7 @@ let new_shared_ops network = {
 let dummy_shared = {
     impl_shared_fullname = "";
     impl_shared_codedname = "";
+    impl_shared_directory = !!CommonComplexOptions.default_incoming;
     impl_shared_val = 0;
     impl_shared_update = 0;
     impl_shared_num = 0;
@@ -244,7 +233,7 @@ let shared_scan_directory shared_dir local_dir =
   if can_share dirname then
     try
       let files = Unix2.list_directory full_dir in
-      lprintf "Sharing sub-directory %s\n" full_dir; 
+       if !verbose_share then lprintf "Sharing sub-directory %s\n" full_dir;
       List.iter (fun file ->
           if file <> "" && file.[0] <> '.' then
             let full_name = Filename.concat full_dir file in
@@ -264,8 +253,7 @@ let shared_scan_directory shared_dir local_dir =
                         Filename2.last_extension full_name)) 
                     strategy.sharing_extensions)
                 then
-                  new_shared dirname shared_dir.shdir_priority
-                    local_name full_name
+                  new_shared shared_dir local_name full_name
               with e -> 
                   lprintf "%s will not be shared (exception %s)\n"
                     full_name (Printexc2.to_string e);
@@ -292,7 +280,7 @@ let _ =
     
 let shared_add_directory shared_dir =
   if shared_dir.shdir_dirname <> "" then begin
-      lprintf "SHARING %s PRIO %d\n" shared_dir.shdir_dirname
+      if !verbose_share then lprintf "SHARING %s PRIO %d\n" shared_dir.shdir_dirname
         shared_dir.shdir_priority;
       shared_add_directory shared_dir ""
     end
@@ -334,16 +322,14 @@ let _ =
       H.iter (fun _ -> incr counter) shareds_by_num;
       Printf.bprintf buf "  shared: %d\n" !counter;
   )
-  
+
 let com_shareds_by_num = shareds_by_num
 let shareds_by_num = ()
 
-(* This won't fit nicely with priorities on upload.
-  Maybe the pririties could be given in another option,
-  and the association would remain independantly of the fact
-  that the directories are currently shared or not. *)
-  
-(*let _ = 
-  Options.set_string_wrappers shared_directories
-    Filepath.semipath_to_string
-    Filepath.string_to_semipath*)
+open CommonComplexOptions
+
+let share_to_commit file =
+  match CommonFile.file_dst_share file with
+    Some sh -> sh
+    | None  -> !!default_incoming
+
