@@ -1,6 +1,7 @@
 type 'a shared_impl = {
   impl_shared_fullname : string;
   impl_shared_codedname : string;
+  impl_shared_directory : CommonTypes.shared_directory;
   mutable impl_shared_val : 'a;
   mutable impl_shared_update : int;
   mutable impl_shared_num : int;
@@ -15,8 +16,6 @@ and 'a shared_ops = {
   mutable op_shared_unshare : 'a -> unit;
 } 
 
-val dirnames_prio : (string * int) list ref
-
 val as_shared : 'a shared_impl -> CommonTypes.shared
 val as_shared_impl : CommonTypes.shared -> 'a shared_impl
 val shared_num : CommonTypes.shared -> int
@@ -31,13 +30,12 @@ val dirnames : (string, string) Hashtbl.t
 val dirname_counter : int ref
 val files_scanned : int ref
 val files_scanned_size : int64 ref
-val new_shared : string -> int -> string -> string -> unit
+val new_shared : CommonTypes.shared_directory -> string -> string -> unit
 val shared_num : CommonTypes.shared -> int
 val shared_fullname : CommonTypes.shared -> string
 val shared_codedname : CommonTypes.shared -> string
+val shared_shared_directory : CommonTypes.shared -> CommonTypes.shared_directory
 val shared_unshare : CommonTypes.shared -> unit
-val shared_dir : CommonTypes.shared option -> string
-val shared_prio : CommonTypes.shared option -> int
 val new_shared_ops : CommonTypes.network -> 'a shared_ops
 val dummy_shared : int shared_impl
 val shared_find : int -> CommonTypes.shared
@@ -49,5 +47,5 @@ val shared_add_directory : CommonTypes.shared_directory -> unit
 val shared_check_files : unit -> unit
 val impl_shared_info : 'a shared_impl -> GuiTypes.shared_info
 val shared_info : CommonTypes.shared -> GuiTypes.shared_info
-
+val share_to_commit : CommonTypes.file -> CommonTypes.shared_directory
   
