@@ -623,7 +623,7 @@ let rec add_shared_file node sh dir_list =
       in
       add_shared_file node sh dir_tail
   
-let add_shared full_name codedname size =
+let add_shared shdir full_name codedname size =
   try
     Hashtbl.find shared_files codedname
   with Not_found ->
@@ -635,6 +635,7 @@ let add_shared full_name codedname size =
           impl_shared_update = 1;
           impl_shared_fullname = full_name;
           impl_shared_codedname = codedname;
+	  impl_shared_directory = shdir;
           impl_shared_size = size;
           impl_shared_id = Md4.random ();
           impl_shared_num = 0;
@@ -823,7 +824,7 @@ let ready_for_upload c =
     
 let add_pending_slot c =
   if client_has_a_slot c then begin
-      lprintf "Avoided inserting an uploader in pending slots!\n";
+      if !verbose_upload then lprintf "Avoided inserting an uploader in pending slots!\n";
     end 
   else 
   if not (Intmap.mem (client_num c) !pending_slots_map) then
@@ -860,25 +861,22 @@ and find_pending_slot () =
   with _ -> ()
 
 let add_pending_slot c =
-  let csh = client_upload c in
-  let cdir = shared_dir csh in
-  let cprio = ref (shared_prio csh) in
-  (* if cdir <> "" then
-    lprintf "Testing cdir %s\n" cdir; *)
-  Intmap.iter (fun _ c -> 
-    let sh = client_upload c in
-    if shared_dir sh = cdir then decr cprio
-  ) !CommonClient.uploaders;
-  (* if cdir <> "" then
-    lprintf "Testing cprio %d\n" !cprio; *)
-  if !cprio > 0 then begin
-    remove_pending_slot c;
-    if client_is_connected c then begin
-      set_client_has_a_slot c true;
-      client_enter_upload_queue c
-    end
-  end else
-    add_pending_slot c
+  match client_upload c with
+   Some csh ->
+    let cdir = shared_shared_directory csh in
+    let cprio = ref cdir.shdir_priority in
+    Intmap.iter (fun _ c -> 
+      match client_upload c with
+       Some sh -> 
+        let dir = shared_shared_directory sh in
+        if dir.shdir_dirname = cdir.shdir_dirname then decr cprio
+       | None  -> ()
+    ) !CommonClient.uploaders;
+    if !cprio > 0 then begin
+      give_a_slot c
+    end else
+      add_pending_slot c
+   | None   -> add_pending_slot c
 
 let static_refill_upload_slots () =
   let len = Intmap.length !CommonClient.uploaders in
@@ -1034,4 +1032,4 @@ let _ =
       buf_int64 buf !total_shared;
       buf_int64 buf !total_uploaded;
       
-  )
\ No newline at end of file
+  )
