@@ -19,8 +19,8 @@
 
 exception FromGuiMessageNotImplemented
 
-val to_gui : int array -> int -> string -> GuiProto.to_gui
-val from_gui : int array -> int -> string -> GuiProto.from_gui
+val to_gui : int array -> (string -> string) -> int -> string -> GuiProto.to_gui
+val from_gui : int array -> (string -> string) -> int -> string -> GuiProto.from_gui
   
 val gui_cut_messages : (int -> string -> unit) -> TcpBufferedSocket.t -> 'a -> unit
   
