@@ -43,13 +43,17 @@ open CommonGlobals
   
 module P = GuiProto
   
+let charset_to_gui s = s
+
+let charset_from_gui s = s
+
 let binary_gui_send gui t =
   match gui.gui_sock with
     None -> 
       Fifo.put core_gui_fifo t
   | Some sock ->
       try
-        GuiEncoding.gui_send (GuiEncoding.to_gui gui.gui_proto_to_gui_version) 
+        GuiEncoding.gui_send (GuiEncoding.to_gui gui.gui_proto_to_gui_version charset_to_gui ) 
         sock t
       with UnsupportedGuiMessage -> 
 (* the message is probably not supported by this GUI *)
@@ -1066,8 +1070,8 @@ let gui_handler t event =
         TcpBufferedSocket.set_reader sock (GuiDecoding.gui_cut_messages
             (fun opcode s ->
               try
-                let m = GuiDecoding.from_gui gui.gui_proto_from_gui_version 
-                  opcode s in
+                let m = GuiDecoding.from_gui gui.gui_proto_from_gui_version
+                charset_from_gui opcode s in
                 gui_reader gui m sock;
               with GuiDecoding.FromGuiMessageNotImplemented -> ()
           ));
@@ -1275,15 +1279,16 @@ let install_hooks () =
       let prefix = r.network_prefix()  in
       List.iter (fun opfile ->
           iter_file (fun o ->
+                option_hook o (fun _ ->
+                    with_guis (fun gui ->
               try
                 let oo = strings_of_option o in
                 let oo = { oo with
                     option_name = Printf.sprintf "%s%s" prefix oo.option_name
                   } in
-                option_hook o (fun _ ->
-                    with_guis (fun gui ->
-                        gui_send gui (P.Options_info [oo])))
-              with _ -> ()
+                      gui_send gui (P.Options_info [oo])
+                    with _ -> ())
+                )
           ) opfile)
       r.network_config_file 
       )      
