@@ -49,7 +49,6 @@ module DP500(M: sig
 
       module CommonOptions : sig 
           
-          val incoming_directory : string Options.option_record
           val temp_directory : string Options.option_record
           val dp500_directory : string Options.option_record
           val allowed_ips : Ip.t list Options.option_record
@@ -148,11 +147,11 @@ let good_file kind is_dir file =
     if !verbose_dp500 then lprintf "Extension [%s]\n" ext;
     List.mem ext
     (match String.lowercase kind with
-        "video" -> [ ".avi"; ".mpg"; ".mpeg" ]
-      | "audio" -> [ ".mp3"; ".ogg"; ".wma" ]
-      | "picture" -> [ ".jpg"; ".jpeg"; ".jpe"; ".gif"; ".tiff" ]
+        "video" -> [ ".avi"; ".mpg"; ".mpeg"; ".ogm"; ".divx" ]
+      | "audio" -> [ ".mp3"; ".ogg"; ".wma"; ".wav" ]
+      | "picture" -> [ ".jpg"; ".jpeg"; ".jpe"; ".gif"; ".tiff"; ".png" ]
       | kind -> 
-          if !verbose_dp500 then lprintf "Unknow kind [%s]\n" kind;
+          if !verbose_dp500 then lprintf "Unknown kind [%s]\n" kind;
           []))
 
 (*************************************************************************)
@@ -279,9 +278,6 @@ let exec_command telnet line sock =
           
           in
           
-          let mlnet_incoming_string = "MLNET INCOMING" in
-          let mlnet_incoming_string_len = String.length mlnet_incoming_string in
-          
           let mlnet_temp_string = "MLNET TEMP" in
           let mlnet_temp_string_len = String.length mlnet_temp_string in
           
@@ -290,23 +286,11 @@ let exec_command telnet line sock =
             | "VIDEO" -> 
 
                 if argument = "" then begin
-                    if !!incoming_directory <> "" then
-                      write_string sock (Printf.sprintf "%s|%s|1|\r\n"
-                          mlnet_incoming_string mlnet_incoming_string
-                      );
-                    
                     if !!temp_directory <> "" then
                       write_string sock (Printf.sprintf "%s|%s|1|\r\n"
                           mlnet_temp_string mlnet_temp_string 
                       );
                     end;
-                if !!incoming_directory <> "" &&
-                  String2.starts_with argument mlnet_incoming_string then
-                    list_directory !!incoming_directory
-                      (String.sub argument mlnet_incoming_string_len
-                        (String.length argument - mlnet_incoming_string_len))
-                  
-                else
                 if !!temp_directory <> "" &&
                   String2.starts_with argument mlnet_temp_string then
                     !!temp_directory,
