@@ -368,8 +368,13 @@ used. For example, we can add new we_binfos... *)
     
 
 (**** CREATE DIRS   ****)
+
+  if List.length !!CommonComplexOptions.shared_directories = 0 then
+  begin
+    restore_default CommonComplexOptions.shared_directories;
+    Unix2.safe_mkdir (List.hd !!CommonComplexOptions.shared_directories).shdir_dirname
+  end;
   
-  Unix2.safe_mkdir !!incoming_directory;
   Unix2.safe_mkdir "searches";
   Unix2.safe_mkdir !!temp_directory
 
@@ -442,12 +447,6 @@ let _ =
   add_infinite_timer 3600. hourly_timer;
   add_infinite_timer 0.1 CommonUploads.upload_download_timer;
 
-  CommonShared.shared_add_directory {
-    shdir_dirname = !!incoming_directory;
-    shdir_priority = !!incoming_directory_prio;
-    shdir_strategy = "only_directory";
-    shdir_networks = [];
-  };
   List.iter CommonShared.shared_add_directory !!CommonComplexOptions.shared_directories;  
   
   add_infinite_timer 1800. (fun timer ->
@@ -470,7 +469,7 @@ let _ =
   
 
   add_init_hook (fun _ ->
-      if not !gui_included then
+      if not !gui_included  && ( !!start_gui || !!ask_for_gui ) then
       (try
           let _ = Sys.getenv("DISPLAY") in
           if !!start_gui && Sys.file_exists !!mldonkey_gui then
