@@ -143,7 +143,7 @@ let gui_send t =
       lprintf "Message not sent since not connected";
       lprint_newline ();
   | Some sock ->
-      GuiEncoding.gui_send (GuiEncoding.from_gui !gui_protocol_used) sock t
+      GuiEncoding.gui_send (GuiEncoding.from_gui !gui_protocol_used (fun s -> s)) sock t
       
 let _ = 
   (try Options.load mldonkey_gui_ini with
