@@ -90,42 +90,42 @@ let current_uploaders = ref ([] : BTTypes.client list)
 let connect_tracker file event f =
   let url = file.file_tracker in
   
-  let args,must_check_delay, downloaded = 
+  let args,must_check_delay, downloaded, left = 
     
     match file.file_swarmer with
       None -> 
-        [("event", "completed")],true,(Int64.of_int 0)
-    | Some swarmer ->
-        lprintf "Connect tracker.........\n";
-        let downloaded = Int64Swarmer.downloaded swarmer in
+       begin 
         match event with
-        | "completed" -> 
-(* ok, when downloading, "completed" shoud be sent immediatly! But
-when sharing, only every interval *)
-            
-            if file_state file = FileDownloading then
-              
-              [("event", "completed")],false,(Int64Swarmer.downloaded swarmer)
-            else
-              [("event", "completed")],true,(Int64.of_int 0)
-        | "started" -> [("event", "started")],true,(Int64Swarmer.downloaded swarmer)
-        | "stopped" -> [("event", "stopped")],false,(Int64Swarmer.downloaded swarmer)
-        | _ -> [],true,(Int64Swarmer.downloaded swarmer)
+        | "started" -> [("event", "started")],true,(Int64.of_int 0),(Int64.of_int 0)
+        | "stopped" -> [("event", "stopped")],false,(Int64.of_int 0),(Int64.of_int 0)
+        | _ -> [],true,(Int64.of_int 0),(Int64.of_int 0)
+       end
+    | Some swarmer ->
+        let local_downloaded = Int64Swarmer.downloaded swarmer in
+        match event with
+        | "completed" -> [("event", "completed")],false,(Int64.of_int 0),(Int64.of_int 0)
+        | "started" -> [("event", "started")],true,local_downloaded,((file_size file) -- local_downloaded)
+        | "stopped" -> [("event", "stopped")],false,local_downloaded,((file_size file) -- local_downloaded)
+        | _ -> [],true,local_downloaded,((file_size file) -- local_downloaded)
   in
   if (not must_check_delay) || (file.file_tracker_last_conn + 
         file.file_tracker_interval 
         < last_time ())
   then
+    begin
     let args = 
       ("info_hash", Sha1.direct_to_string file.file_id) ::
       ("peer_id", Sha1.direct_to_string !!client_uid) ::
       ("port", string_of_int !!client_port) ::
       ("uploaded", Int64.to_string file.file_uploaded) ::
       ("downloaded", Int64.to_string downloaded) ::
-      ("left", Int64.to_string ((file_size file) -- downloaded)) ::
+      ("left", Int64.to_string left) ::
       ("compact","1") ::
       args
     in  
+
+    let args = if !!numwant > -1 then ("numwant", string_of_int !!numwant) :: args else args in
+
     let module H = Http_client in
     let r = {
         H.basic_request with
@@ -135,15 +135,16 @@ when sharing, only every interval *)
         Printf.sprintf "MLdonkey/%s" Autoconf.current_version;
       } in
     
-        lprintf "Request sent to tracker\n";
+        if !verbose_msg_servers then lprintf "Request sent to tracker, file: %s\n" file.file_name;
         H.wget r 
           (fun fileres -> 
             file.file_tracker_last_conn <- last_time ();
             file.file_tracker_connected <- true;        
             f fileres
         )
+    end
       else
-        lprintf "Request NOT sent to tracker - remaning: %d\n" (file.file_tracker_interval - (last_time () - file.file_tracker_last_conn))
+        if !verbose_msg_servers then lprintf "Request NOT sent to tracker - remaning: %d file: %s\n" (file.file_tracker_interval - (last_time () - file.file_tracker_last_conn)) file.file_name
         
 
 (** In this function we decide which peers will be 
@@ -152,31 +153,17 @@ when sharing, only every interval *)
   for clients that are in next list (and not in current) 
 *)
 let recompute_uploaders () =
-  
-(*  lprintf "recompute_uploaders\n"; *)
-  
-  next_uploaders := choose_best_downloaders
-    (List.filter
-      (fun f ->  file_state f = FileDownloading )
-    !current_files );
-  
-  next_uploaders := !next_uploaders @ (choose_best_uploaders
-      (List.filter
-        (fun f ->  file_state f = FileShared )
-      !current_files )); 
-
-(*  lprintf "next_uploaders: %d \n" (List.length !next_uploaders); *)
-  
-(*Send choke if a current_uploader is not in next_uploaders*)      
+  if !verbose_upload then lprintf "recompute_uploaders\n";
+  next_uploaders := choose_uploaders current_files;
+  (*Send choke if a current_uploader is not in next_uploaders*)      
   List.iter ( fun c -> if ((List.mem c !next_uploaders)==false) then
         begin
           set_client_has_a_slot (as_client c) false;
-(*we will let him finish is download and 
-			choke him on next_request*)
+          (*we will let him finish his download and choke him on next_request*)
         end
   ) !current_uploaders;
-
-(*don't send Choke if new uploader is already an uploaders *)            
+  
+  (*don't send Choke if new uploader is already an uploaders *)            
   List.iter ( fun c -> if ((List.mem c !current_uploaders)==false) then
         begin
           set_client_has_a_slot (as_client c) true;
@@ -188,7 +175,7 @@ let recompute_uploaders () =
         end
   ) !next_uploaders;
   current_uploaders := !next_uploaders
-  
+
 
 (****** Fabrice: why are clients which are disconnected removed ???
   These clients might still be useful to reconnect to, no ? *)
@@ -290,13 +277,14 @@ let disconnect_clients file =
   @param file the finished file
 *)         
 let download_finished file = 
-  if List.memq file !current_files then begin      
-    (*CommonComplexOptions.file_completed*)    
+  if List.memq file !current_files then begin
+      connect_tracker file "completed" (fun _ -> ()); (*must be called before swarmer gets removed from file*)
+      connect_tracker file "stop" (fun _ -> ()); (*we do that one here, because afaik we when starting seeding we send start again and until that happens we refuse incoming connections for this file*)
+      (*CommonComplexOptions.file_completed*)
       file_completed (as_file file);
       remove_file file;
       disconnect_clients file;
-      connect_tracker file "stopped" (fun _ -> ());
-(* start sharing! - Copy file from torrents/download to torrent/seeded *)
+      (* start sharing! - Copy file from torrents/download to torrent/seeded *)
       try
         let s = File.to_string file.file_torr_fname in  
         if String.length s = 0 then raise Not_found;
@@ -422,10 +410,15 @@ let rec client_parse_header counter cc init_sent gconn sock
       match !cc with 
         None ->
           let c = new_client file peer_id (TcpBufferedSocket.host sock) in
-          lprintf "CLIENT %d: incoming CONNECTION\n" (client_num c);
+          if !verbose_connect then lprintf "CLIENT %d: incoming CONNECTION\n" (client_num c);
           cc := Some c;
           c
-      | Some c -> c
+      | Some c -> 
+         (* client could have had Sha1.null as peer_id/uid *)
+         if c.client_uid <> peer_id then 
+          c.client_software <- (parse_software (Sha1.direct_to_string peer_id));
+          c
+
 (*          if c.client_uid <> peer_id then begin
               lprintf "Unexpected client by UID\n";
               let ccc = new_client file peer_id (TcpBufferedSocket.host sock) in
@@ -491,16 +484,19 @@ let rec client_parse_header counter cc init_sent gconn sock
 
 (*    send_client c Unchoke;  *)
     
-    set_rtimeout sock 300.;
+    set_rtimeout sock !!client_timeout;
 (*Once parse succesfully we define the function
     client_to_client to be the function used when a message
     is read*)
     gconn.gconn_handler <- Reader (fun gconn sock ->
-        bt_handler bt_parser (client_to_client c) sock
+        bt_handler bt_parser (client_to_client c) c.client_software sock
     );
     
     ()
-  with e ->
+  with
+    | Not_found ->
+      lprintf "BT: requested file not shared [%s]\n" (Sha1.to_hexa file_id)
+    | e ->
       lprintf "Exception %s in client_parse_header\n" (Printexc2.to_string e);
       close sock (Closed_for_exception e);
       raise e
@@ -778,6 +774,10 @@ and client_to_client c sock msg =
 (* Check if the client is still interesting for us... *)
         check_if_interesting file c
     
+    | PeerID p ->
+      c.client_software <- (parse_software p);
+      c.client_uid <- Sha1.direct_of_string p
+
     | BitField p ->
 (*A bitfield is a summary of what a client have*)
         
@@ -830,7 +830,8 @@ and client_to_client c sock msg =
           (try get_from_client sock c with _ -> ())
         done*)
         end;
-        if c.client_incoming then send_bitfield c;
+        (*a bitfield must only be sent after the handshake and befor everything else: NOT here
+        if c.client_incoming then send_bitfield c;*)
     
     | Have n ->
 (*A client can send a Have without sending a Bitfield*)
@@ -882,11 +883,13 @@ and client_to_client c sock msg =
         begin
           set_client_state (c) (Connected (-1));
 (*remote peer will clear the list of range we sent*)
-          let up = match c.client_uploader with
-              None -> assert false
-            | Some up -> up in 
-          
-          Int64Swarmer.clear_uploader_ranges up;
+          begin
+          match c.client_uploader with
+              None ->
+                if !verbose_msg_clients then lprintf "BT: Choke send, but no client bitmap\n"
+            | Some up -> 
+          Int64Swarmer.clear_uploader_ranges up
+          end;
           c.client_ranges_sent <- [];
           c.client_range_waiting <- None;
           c.client_choked <- true;
@@ -949,10 +952,18 @@ be put in a fifo and dequeud according to
 @param c The client we must connect
 *)      
 let connect_client c =
-  if can_open_connection connection_manager then
+  if can_open_connection connection_manager && 
+	(let (ip,port) = c.client_host in match Ip_set.match_ip !Ip_set.bl ip with
+	     None -> true
+	   | Some br ->
+	       if !verbose_connect then
+		 lprintf "%s:%d blocked: %s\n"
+		   (Ip.to_string ip) port br.blocking_description;
+	       false)		
+  then
   match c.client_sock with
     NoConnection ->
-      
+    
       let token =
         add_pending_connection connection_manager (fun token ->
             try
@@ -1041,7 +1052,7 @@ let listen () =
 	      the TcpServerSocket means that a new client try 
 		to connect to us*)
 	      let ip = (Ip.of_inet_addr from_ip) in 
-              lprintf "CONNECTION RECEIVED FROM %s\n"
+              if !verbose_sources > 1 then lprintf "CONNECTION RECEIVED FROM %s\n"
                 (Ip.to_string (Ip.of_inet_addr from_ip))
               ; 
 (*Reject this connection if we don't want
@@ -1132,8 +1143,9 @@ let resume_clients file =
   Hashtbl.iter (fun _ c ->
       try
         match c.client_sock with 
-        | Connection sock -> 
-            lprintf "RESUME: Client is already conencted\n";
+        | Connection sock -> ()
+            (*i think this one is not realy usefull for debugging
+			lprintf "RESUME: Client is already connected\n"; *)
         | _ ->
             (try 
 	       (*test if we can connect client according to the its 
@@ -1162,15 +1174,15 @@ let resume_clients file =
   we really ask sources to the tracker
 *)
 let get_sources_from_tracker file = 
-  lprintf "get_sources_from_tracker\n";
   let f filename = 
 (*This is the function which will be called by the http client
 for parsing the response*)
-    lprintf "Filename %s\n" filename;
     let v = Bencode.decode (File.to_string filename) in
     
-    lprintf "Received: %s\n" (Bencode.print v);
-    let interval = ref 600 in
+	if !verbose_sources > 1 then lprintf "Received: %s\nfor file: %s" (Bencode.print v) filename;
+    file.file_tracker_interval <- 600;
+	file.file_last_clients_num_from_tracker <- 0;
+	begin
     match v with
       Dictionary list ->
         List.iter (fun (key,value) ->
@@ -1181,14 +1193,15 @@ for parsing the response*)
                 List.iter (fun v ->
                     match v with
                       Dictionary list ->
+					    file.file_last_clients_num_from_tracker <- file.file_last_clients_num_from_tracker+1;
                         let peer_id = ref Sha1.null in
                         let peer_ip = ref Ip.null in
                         let port = ref 0 in
-                        
+
                         List.iter (fun v ->
                             match v with
                               String "peer id", String id -> 
-                                peer_id := Sha1.direct_of_string id
+                                peer_id := Sha1.direct_of_string id;
                             | String "ip", String ip ->
                                 peer_ip := Ip.of_string ip
                             | String "port", Int p ->
@@ -1209,8 +1222,8 @@ for parsing the response*)
 				  false)
 			then
                           let c = new_client file !peer_id (!peer_ip,!port)
-                          in
-                          lprintf "Received %s:%d\n" (Ip.to_string !peer_ip)
+						  in
+                          if !verbose_sources > 1 then lprintf "Received %s:%d\n" (Ip.to_string !peer_ip)
                           !port;
                           ()
                     
@@ -1226,6 +1239,7 @@ for parsing the response*)
 		    and port = get_int16 s (pos+4) 
 		    in 
 		      ignore( new_client file Sha1.null (ip,port));
+			  file.file_last_clients_num_from_tracker <- file.file_last_clients_num_from_tracker+1;
 		      iter_comp s (pos+6) l
 		in
 		  iter_comp p 0 (String.length p)
@@ -1236,18 +1250,40 @@ for parsing the response*)
         resume_clients file
     
     | _ -> assert false    
+	end;
+	if !verbose_sources > 0 then lprintf "get_sources_from_tracker: got %i sources for file %s\n" file.file_last_clients_num_from_tracker file.file_name
   in
   let event = 
-    if file.file_tracker_connected then "" else
-    match file_state file with
-      FileShared -> "completed"
-    | _ -> "started" 
+    if file.file_tracker_connected then ""
+    else "started"
   in
-  if file.file_clients_num < !!ask_tracker_threshold then
-    connect_tracker file event f
-    
-    
-  
+  (* if we have too few sources we may ask the tracker before the interval *)
+  if file.file_clients_num < !!ask_tracker_threshold &&
+     (file_state file) == FileDownloading &&
+       (
+        (file.file_last_clients_num_from_tracker > 4 &&
+         file.file_tracker_last_conn + 120 < last_time() &&
+         file.file_clients_num > (file.file_last_clients_num_from_tracker/5*4)
+        ) ||
+        file.file_tracker_last_conn + 600 < last_time() ||
+        (file.file_last_clients_num_from_tracker < 5 &&
+         file.file_tracker_last_conn + 300 < last_time()
+        ) ||
+        not file.file_tracker_connected
+       )
+  then
+    begin
+      if !verbose_msg_servers then
+        lprintf "get_sources_from_tracker: connected:%s last_clients:%i last_conn-last_time:%i file: %s\n"
+            (string_of_bool file.file_tracker_connected) file.file_last_clients_num_from_tracker
+                (file.file_tracker_last_conn - last_time()) file.file_name;
+      file.file_tracker_last_conn <- (last_time() - (file.file_tracker_interval + 1));
+    end;
+    (* We MUST update tracker for upload/download info ... and that we are still alive *)
+  connect_tracker file event f
+
+
+
 
 (** Check to see if file is finished, if not 
   try to get sources for it
@@ -1260,21 +1296,19 @@ let recover_files () =
           (try check_finished swarmer file with e -> ());
           match file_state file with
             FileDownloading ->
-(* This one is useless vvv (it's called in 
-	     get_sources_from_tracker)
-	     (try resume_clients file with _ -> ());*)
-              (try 
-                  get_sources_from_tracker file  
-                with _ -> ())
+                (try get_sources_from_tracker file 
+                    with _ -> ())
           | FileShared ->
-              (try 
-                  connect_tracker file "completed" (fun _ -> ())
-                with _ -> ())
+                (try connect_tracker file "" (fun _ -> ())
+                    with _ -> ())
+          | FilePaused -> 
+                () (*when we are paused we do nothing, not even logging this vvvv*)
           | s -> lprintf "Other state %s!!\n" (string_of_state s)
       ) !current_files
-      
+
+
 let upload_buffer = String.create 100000
-  
+
 
 (**
   Send a Piece message 
@@ -1355,13 +1389,12 @@ let file_resume file =
 *)
 let file_stop file =
     if file.file_tracker_connected then 
-      begin
-	connect_tracker file "stopped" (fun _ -> ());
-	  (*This vvvv must be after after this ^^^^ *)
-	file.file_tracker_connected <- false
-      end
-      
-      
+        connect_tracker file "stopped" (fun _ -> (
+            lprintf "BT-Tracker return: stopped %s\n" file.file_name;
+            file.file_tracker_connected <- false)
+          )
+
+
 (**
   Create the 'hooks'
 *)            
