@@ -89,11 +89,49 @@ let _ =
 (*          try Unix32.rename (file_fd file) (new_name) with _ -> () *)
         end	
   );
-  file_ops.op_file_print_sources_html_header <- (fun file buf info ->
+
+  file_ops.op_file_print_html <- (fun file buf ->
+
+    html_mods_cntr_init ();
+
+    Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+    html_mods_td buf [ 
+      ("Filename", "sr br", "Filename");
+      ("", "sr", file.file_name ) ];
+
+    Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+    html_mods_td buf [
+      ("Tracker", "sr br", "Tracker");
+      ("", "sr", file.file_tracker ) ]; 
+
+    Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+    html_mods_td buf [
+      ("Torrent Fname", "sr br", "Torrent Fname");
+      ("", "sr", file.file_torr_fname ) ]; 
+
+    Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+    html_mods_td buf [
+      ("Connect Interval", "sr br", "Con Inverval");
+      ("", "sr", Printf.sprintf "%d" file.file_tracker_interval ) ]; 
+
+    Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+    html_mods_td buf [
+      ("Last Connect", "sr br", "Last Connect");
+      ("", "sr", (string_of_date file.file_tracker_last_conn)) ]; 
+
+  );
+  file_ops.op_file_print_sources_html <- (fun file buf ->
+
+    if Hashtbl.length file.file_clients > 0 then begin
+
+      let chunks = (match file.file_swarmer with
+          None -> "" | Some swarmer ->
+            Int64Swarmer.verified_bitmap swarmer) in
       
       html_mods_table_header buf "sourcesTable" "sources al" [ 
         ( "1", "srh br ac", "Client number", "Num" ) ; 
         ( "0", "srh br", "Client UID", "UID" ) ; 
+        ( "0", "srh br", "Client software", "Soft" ) ; 
         ( "0", "srh", "IP address", "IP address" ) ; 
         ( "0", "srh br ar", "Port", "Port" ) ; 
         ( "1", "srh ar", "Total UL bytes to this client for all files", "UL" ) ; 
@@ -101,6 +139,19 @@ let _ =
         ( "1", "srh ar", "Interested [T]rue, [F]alse", "I" ) ; 
         ( "1", "srh ar", "Choked [T]rue, [F]alse", "C" ) ; 
         ( "1", "srh br ar", "Allowed to write", "A" ) ; 
+
+        ( "1", "srh ar", "Interesting [T]rue, [F]alse", "I" );
+        ( "1", "srh ar", "Already sent interested [T]rue, [F]alse", "A" );
+        ( "1", "srh br ar", "Already sent not interested [T]rue, [F]alse", "N" );
+
+        ( "1", "srh ar", "Good [T]rue, [F]alse", "G" );
+        ( "1", "srh ar", "Incoming [T]rue, [F]alse", "I" );
+        ( "1", "srh br ar", "Registered bitfield [T]rue, [F]alse", "B" );
+
+        ( "1", "srh ar", "Optimist Time", "O" );
+        ( "1", "srh ar", "Last optimist", "L.Opt" );
+        ( "1", "srh br ar", "Num try", "N" );
+
 (* 
 		( "0", "srh", "Bitmap (absent|partial|present|verified)", (colored_chunks 
         (Array.init (String.length info.G.file_chunks)
@@ -108,14 +159,68 @@ let _ =
 *)
         ( "1", "srh ar", "Number of full chunks", (Printf.sprintf "%d"
               (String.length (String2.replace
-                (String2.replace info.G.file_chunks '0' "") '1' "")) )) ]      
-  )
+                (String2.replace chunks '0' "") '1' "")) )) ]      ;
+
+      Hashtbl.iter (fun _ c ->
+          Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr());
   
+        html_mods_td buf [
+          ("", "sr br ar", Printf.sprintf "%d" (client_num c));
+          ("", "sr br", (Sha1.to_string c.client_uid));
+          ("", "sr br", c.client_software);
+          ("", "sr", (Ip.to_string (fst c.client_host)));
+          ("", "sr br ar", Printf.sprintf "%d" (snd c.client_host));
+          ("", "sr ar", (size_of_int64 c.client_uploaded));
+          ("", "sr ar br", (size_of_int64 c.client_downloaded));
+          ("", "sr", (if c.client_interested then "T" else "F"));
+          ("", "sr", (if c.client_choked then "T" else "F"));
+          ("", "sr br ar", (Int64.to_string c.client_allowed_to_write));
+(* This is way too slow for 1000's of chunks on a page with 100's of sources 
+    ("", "sr", (CommonFile.colored_chunks (Array.init (String.length c.client_bitmap)
+       (fun i -> (if c.client_bitmap.[i] = '1' then 2 else 0)) )) );
+*)
+          ("", "sr", (if c.client_interesting then "T" else "F"));
+          ("", "sr", (if c.client_alrd_sent_interested then "T" else "F"));
+          ("", "br sr", (if c.client_alrd_sent_notinterested then "T" else "F"));
+
+          ("", "sr", (if c.client_good then "T" else "F"));
+          ("", "sr", (if c.client_incoming then "T" else "F"));
+          ("", "br sr", (if c.client_registered_bitfield then "T" else "F"));
+
+          ("", "sr", Printf.sprintf "%d" c.client_optimist_time);
+          ("", "ar sr", string_of_date c.client_last_optimist);
+          ("", "br sr", Printf.sprintf "%d" c.client_num_try);
+
+          ("", "sr ar", (let fc = ref 0 in 
+              (match c.client_bitmap with
+                  None -> ()
+                | Some bitmap ->
+                    String.iter (fun s -> if s = '1' then incr fc) bitmap );
+              (Printf.sprintf "%d" !fc) ) ) ];
+
+        Printf.bprintf buf "\\</tr\\>";
+
+      ) file.file_clients;
+
+      Printf.bprintf buf "\\</table\\>\\</div\\>\\<br\\>";
+
+   end;
+        
+  );
+  file_ops.op_file_check <- (fun file ->
+    lprintf "Checking chunks of %s\n" file.file_name;
+    match file.file_swarmer with
+      None ->
+        lprintf "verify_chunks: no swarmer to verify chunks"
+    | Some swarmer ->
+        Int64Swarmer.verify_all_chunks swarmer true
+  );
   
 module P = GuiTypes
   
 let _ =
   file_ops.op_file_cancel <- (fun file ->
+      BTClients.file_stop file;
       remove_file file;
       BTClients.disconnect_clients file;
 (*
@@ -177,12 +282,12 @@ let load_torrent_string s =
   file
   
 let load_torrent_file filename =
+  lprintf "BTInteractive.load_torrent_file %s\n" filename;
   let s = File.to_string filename in  
   
   let download_filename = Filename.concat downloads_directory
       (Filename.basename filename) in
   File.from_string download_filename s;
-  lprintf "BTInteractive.load_torrent_file %s\n" download_filename;
   let file = load_torrent_string s in
   file.file_torr_fname <- download_filename
 
@@ -190,10 +295,10 @@ let parse_tracker_reply file filename =
 (*This is the function which will be called by the http client
 for parsing the response*)
 (* Intrested only in interval*)
-  lprintf "Filename %s\n" filename;
+  if !verbose_msg_servers then lprintf "Filename %s\n" filename;
   let v = Bencode.decode (File.to_string filename) in
 
-  lprintf "Received: %s\n" (Bencode.print v);
+  if !verbose_msg_servers then lprintf "Received: %s\n" (Bencode.print v);
   file.file_tracker_interval <- 600;
   match v with
     Dictionary list ->
@@ -201,7 +306,7 @@ for parsing the response*)
           match (key, value) with
             String "interval", Int n ->
               file.file_tracker_interval <- Int64.to_int n;
-              lprintf ".. interval %d ..\n" file.file_tracker_interval
+              if !verbose_msg_servers then lprintf ".. interval %d ..\n" file.file_tracker_interval
           | _ -> ()
       ) list;
   | _ -> assert false
@@ -211,8 +316,9 @@ let try_share_file filename =
   let ss = filename in
   let s = File.to_string filename in  
   let file_id, torrent = BTTracker.decode_torrent s in
-  
-  let filename = Filename.concat (Filename.concat !!incoming_directory !!commit_in_subdir) torrent.torrent_name in
+
+  let incoming_directory = !!default_incoming.shdir_dirname in
+  let filename = Filename.concat (Filename.concat incoming_directory !!commit_in_subdir) torrent.torrent_name in
 
   if Sys.file_exists filename then 
     let file_u = 
@@ -222,7 +328,7 @@ let try_share_file filename =
       else
         Unix32.create_rw filename
     in
-    lprintf "Sharing file %s\n" filename;
+    if !verbose_msg_servers then lprintf "Sharing file %s\n" filename;
     
     let file = new_file file_id torrent.torrent_name 
         torrent.torrent_length 
@@ -241,16 +347,16 @@ let try_share_file filename =
     Int64Swarmer.set_verified_bitmap swarmer verified;
 *)
     
-    lprintf "......\n";
+    if !verbose_msg_servers then lprintf "......\n";
     file.file_files <- torrent.torrent_files;
-    BTClients.connect_tracker file "completed" 
+    BTClients.connect_tracker file "started" 
       (parse_tracker_reply file)
 
     
 (* Call one minute after start, and then every 20 minutes. Should 
   automatically contact the tracker. *)    
 let share_files _ =
-  lprintf "BTInteractive.share_files\n";
+  if !verbose_share then lprintf "BTInteractive.share_files\n";
   List.iter (fun dir ->
       let filenames = Unix2.list_directory dir in
       List.iter (fun file ->
@@ -262,33 +368,45 @@ let share_files _ =
   List.iter (fun file ->
     if not (Sys.file_exists file.file_torr_fname) && file_state file = FileShared then
     begin
-      lprintf "Removing torrent share for %s\n" file.file_torr_fname;
+      if !verbose_share then lprintf "Removing torrent share for %s\n" file.file_torr_fname;
       BTClients.file_stop file;
       remove_file file;
       BTClients.disconnect_clients file
     end
   ) !copy_shfiles
-    
+
+let valid_torrent_extension url = 
+  let ext = String.lowercase (Filename2.last_extension url) in
+    if ext = ".torrent" || ext = ".tor" then true else false
+
+let get_regexp_string text r = 
+  ignore (Str.search_forward r text 0);
+  let a = Str.group_beginning 1 in
+  let b = Str.group_end 1 in
+    String.sub text a (b - a)
   
 let _ =
   network.op_network_parse_url <- (fun url ->
-      let ext = String.lowercase (Filename2.last_extension url) in
-      lprintf "Last extension: %s\n" ext;
-      if ext = ".torrent" || ext = ".tor" then
-        try
-          lprintf "Trying to load %s\n" url;
-          load_torrent_file url;
-          true
-        with e ->
-            lprintf "Exception %s while loading\n" (Printexc2.to_string e);
+  (*lprintf "BT.op_network_parse_url";*)
+  let location_regexp = "Location: \\(.*\\)" in
+  try let real_url = get_regexp_string url (Str.regexp location_regexp) in
+    if (valid_torrent_extension real_url) || 
+      (String2.contains url "Content-Type: application/x-bittorrent") then (
+	    let u = Url.of_string real_url in
             let module H = Http_client in
-	    let u = Url.of_string url in
             let r = {
                 H.basic_request with
                 H.req_url = u;
                 H.req_proxy = !CommonOptions.http_proxy;
                 H.req_user_agent = 
                 Printf.sprintf "MLdonkey/%s" Autoconf.current_version;
+	H.req_referer = (
+           let referers = !!BTOptions.referers in
+           let (rule_search,rule_value) = try (List.find(fun (rule_search,rule_value) ->
+                           Str.string_match (Str.regexp rule_search) real_url 0
+                           ) referers )
+                           with Not_found -> ("",real_url) in
+                Some (Url.of_string rule_value) ); 
 		H.req_headers = try
 		  let cookies = List.assoc u.Url.server !!BTOptions.cookies in
 		  [ ( "Cookie", List.fold_left (fun res (key, value) ->
@@ -305,10 +423,21 @@ let _ =
             lprintf "wget started\n";
             
             true
-      else
-        false
-  )
-  
+	    )
+    else
+      false
+    with Not_found -> 
+	if (valid_torrent_extension url) then
+	    try
+		load_torrent_file url;
+		true
+	    with e ->
+		lprintf "Exception %s while loading\n" (Printexc2.to_string e);
+		false
+	else
+	    false
+    )
+
 let _ = (
     client_ops.op_client_info <- (fun c ->
         let (ip,port) = c.client_host in
@@ -326,11 +455,11 @@ let _ = (
           P.client_rating = 0;
           P.client_chat_port = 0 ;
           P.client_connect_time = BasicSocket.last_time ();
-          P.client_software = "";
+          P.client_software = c.client_software;
           P.client_emulemod = "";
           P.client_downloaded = c.client_downloaded;
           P.client_uploaded = c.client_uploaded;
-          P.client_upload = None;
+          P.client_upload = Some (c.client_file.file_name);
 (*          P.client_sock_addr = (Ip.to_string ip); *)
         }
     );
@@ -347,29 +476,6 @@ let _ = (
           cinfo.GuiTypes.client_name
           (Sha1.to_string c.client_uid)
     );
-    client_ops.op_client_bprint_html <- (fun c buf file ->
-        
-        html_mods_td buf [
-          ("", "sr br ar", Printf.sprintf "%d" (client_num c));
-          ("", "sr br", (Sha1.to_string c.client_uid));
-          ("", "sr", (Ip.to_string (fst c.client_host)));
-          ("", "sr br ar", Printf.sprintf "%d" (snd c.client_host));
-          ("", "sr ar", (size_of_int64 c.client_uploaded));
-          ("", "sr ar br", (size_of_int64 c.client_downloaded));
-          ("", "sr", (if c.client_interested then "T" else "F"));
-          ("", "sr", (if c.client_choked then "T" else "F"));
-          ("", "sr br ar", (Int64.to_string c.client_allowed_to_write));
-(* This is way too slow for 1000's of chunks on a page with 100's of sources 
-		("", "sr", (CommonFile.colored_chunks (Array.init (String.length c.client_bitmap)
-       (fun i -> (if c.client_bitmap.[i] = '1' then 2 else 0)) )) );
-*)
-          ("", "sr ar", (let fc = ref 0 in 
-              (match c.client_bitmap with
-                  None -> ()
-                | Some bitmap ->
-                    String.iter (fun s -> if s = '1' then incr fc) bitmap );
-              (Printf.sprintf "%d" !fc) ) ) ];
-    );
     client_ops.op_client_dprint <- (fun c o file ->
         let info = file_info file in
         let buf = o.conn_buf in
@@ -439,9 +545,7 @@ let _ =
         try_share_file torrent;
         lprintf "4\n";
         ".torrent file generated"
-    ), _s " <filename> : generate the corresponding <filename>.torrent file
-in torrents/tracked/. The file is automatically tracked, and seeded if
-    in incoming/";
+    ), _s " <filename> :\t\tgenerate the corresponding <filename>.torrent file in torrents/tracked/.\n\t\t\t\t\tThe file is automatically tracked, and seeded if in incoming/";
     
     "torrents", "Network/Bittorrent", Arg_none (fun o ->
         
@@ -458,7 +562,7 @@ in torrents/tracked/. The file is automatically tracked, and seeded if
           end
         else
           _s "Tracker not activated (tracker_port = 0)"
-    ), _s " : print all .torrent files on this server";
+    ), _s " :\t\t\t\tprint all .torrent files on this server";
 
     "seeded_torrents", "Network/Bittorrent", Arg_none (fun o ->
 
@@ -468,7 +572,18 @@ in torrents/tracked/. The file is automatically tracked, and seeded if
       ) !current_files;
       _s "done"
 
-    ), _s " : print all sedded .torrent files on this server";
+    ), _s " :\t\t\tprint all sedded .torrent files on this server";
+    
+    "reshare_torrents", "Network/Bittorrent", Arg_none (fun o ->
+        share_files ();
+        _s "done"
+       ), _s " :\t\t\trecheck torrents/* directories for changes";
+    
+    "stop_all_bt", "Network/Bittorrent", Arg_none (fun o ->
+         List.iter (fun file -> BTClients.file_stop file ) !current_files;
+         _s "started sending stops"
+        ), _s " :\t\t\t\tstops all bittorrent downloads, use this if you want to make sure that the stop signal actualy gets to the tracker\n\t\t\t\t\twhen shuting mlnet down, but you have to wait till the stops get to the tracker and not wait too long,\n\t\t\t\t\tso mldonkey reconnects to the tracker :)";
+    
     (*
     "print_torrent", Arg_one (fun filename o ->
 
@@ -489,4 +604,4 @@ let gui_message s =
   | opcode -> failwith (Printf.sprintf "BT: Unknown message opcode %d" opcode)
   
 let _ =
-  network.op_network_gui_message <- gui_message
\ No newline at end of file
+  network.op_network_gui_message <- gui_message
