@@ -92,12 +92,33 @@ let max_uploaders_per_torrent = define_option bittorrent_section ["max_uploaders
     "Maximum number of uploaders for one torrent"
     int_option 5
 
+let max_bt_uploaders = define_option bittorrent_section ["max_bt_uploaders"]
+    "Maximum number of uploaders for bittorrent"
+    int_option 5
+
+(* numwant: Optional. Number of peers that the client would like to receive from the tracker. 
+This value is permitted to be zero. If omitted, typically defaults to 50 peers.   *)
+
+let numwant = define_option bittorrent_section ["numwant"]
+    "Number of peers to request from tracker (Negative # = let tracker decide)"
+    int_option (-1)
+
 let _ =
-  option_hook max_uploaders_per_torrent
-    (fun _ ->
-      if !!max_uploaders_per_torrent < 1 then max_uploaders_per_torrent =:= 5)
+  begin
+    option_hook max_uploaders_per_torrent
+      (fun _ ->
+        if !!max_uploaders_per_torrent < 1 then max_uploaders_per_torrent =:= 5);
+    option_hook max_bt_uploaders
+      (fun _ ->
+        if !!max_bt_uploaders < 0 then max_bt_uploaders =:= 5)
+  end
   
   
 let cookies = define_option bittorrent_section ["cookies"]
     "Cookies send with http request to get .torrent file"
     (list_option (tuple2_option (string_option, list_option (tuple2_option (string_option, string_option))))) []
+
+let referers = define_option bittorrent_section ["referers"]
+    "Referer sent with http request to get .torrent file" 
+    (list_option (tuple2_option (string_option, string_option))) [(".*suprnova.*", "http://www.suprnova.org/")]
+    
