@@ -253,6 +253,7 @@ type msg =
 | Piece of int * int64 * string * int * int
 | Cancel of int64 * int64 * int64
 | Ping  
+| PeerID of string
 
   
 let bt_print msg =
@@ -269,6 +270,7 @@ let bt_print msg =
       lprintf "Piece %d %Ld[%d]\n" index offset len
   | Cancel _ -> lprintf "Cancel\n"
   | Ping   -> lprintf "Ping\n"
+  | PeerID s -> lprintf "PeerID [%s]\n" (String.escaped s)
     
 let bt_parser opcode m = 
   match opcode with
@@ -280,25 +282,72 @@ let bt_parser opcode m =
   | 5 -> BitField m
   | 6 -> Request (get_int m 0, get_uint64_32 m 4, get_uint64_32 m 8)
   | 7 -> Piece (get_int m 0, get_uint64_32 m 4, m, 8, String.length m - 8)
+  | -1 -> PeerID m
   | _ -> raise Not_found
   
-let bt_handler parse_fun handler sock =
+let bt_handler parse_fun handler client_software sock =
   try
     let b = TcpBufferedSocket.buf sock in
+    let first = ref (
+      if client_software = "NULL" then 0
+      else 1
+    ) in
 (*
     lprintf "BT HANDLER\n";
 dump (String.sub b.buf b.pos b.len);
   *)
     try
-      while not (closed sock) && b.len >= 4 do
-(*        lprintf "BT RECEIVED %d %d/%d \n" b.pos b.len (String.length b.buf); *)
+      (*it is possible to connection be closed, BEFORE we use all data in buffer?*)
+      while b.len >= 4 do
+        (*lprintf "BT RECEIVED %d %d/%d \n" b.pos b.len (String.length b.buf);*)
+        (*hack: if we can guess a peer_id came in late, throw it away and parse the rest as usual
+                otherwise everything we got is thrown away*)
+        (* oo boy, o boy, what a fuck up!! What if clientid is [\0\0\0\0\0\0\0\0 ....]?? *)
         let msg_len = get_int b.buf b.pos in
-        if msg_len < 0 then
+        if !first == 0 then
           begin
-            lprintf "BT: Unknown message dropped!!\n";
-            dump (String.sub b.buf b.pos b.len);
-            buf_used b b.len;
+            if b.len < 20 then
+              begin
+                if not (closed sock) then
+                  begin
+                    if !verbose_msg_clients then
+                      begin
+                        lprintf "BT: Received only %d in first packet, waiting more data\n" b.len;
+                        dump (String.sub b.buf b.pos b.len);
+                      end;
+                    raise Not_found;
+                  end;
+                if !verbose_msg_clients then
+                  begin
+                    lprintf "BT: First message too small (%d), disconnecting\n" b.len;
+                    dump (String.sub b.buf b.pos b.len);
+                  end;
+                buf_used b b.len;
+              end
+            else
+              begin
+                let payload = String.sub b.buf b.pos 20 in
+                if !verbose_msg_clients then 
+                    lprintf "BT: late peer_id  data_len: %i  peer_id: [%s]\n" b.len (String.escaped payload);
+                let p = parse_fun (-1) payload in
+                buf_used b 20;
+                try
+                  handler sock p;
+                  first := 1
+                with e ->
+                  lprintf "Exception %s in BTProtocol.parse_fun\n"
+                    (Printexc2.to_string e);
+                  lprintf "      PeerID\n";
+                  dump payload
+              end
           end
+        else if msg_len < 0 || msg_len > 20000 then
+              begin
+                lprintf "BT: Unknown message dropped!! data_len: %i msg_len: %i\n" b.len msg_len;
+                dump (String.sub b.buf b.pos b.len);
+                buf_used b b.len;
+                close sock Closed_by_user;
+              end
         else
           begin
             if b.len >= 4 + msg_len then
@@ -451,6 +500,7 @@ let send_client client_sock msg =
               Buffer.add_substring buf s pos len
           
           | Cancel _ -> ()
+          | PeerID _ -> ()
           | Ping -> ()          
         end;
         let s = Buffer.contents buf in
