@@ -58,7 +58,7 @@ torrents/: for BitTorrent
   *)
   
 let decode_torrent s =
-  lprintf ".torrent file loaded\n";
+  if !verbose_msg_servers then lprintf ".torrent file loaded\n";
 (*            lprintf "Loaded: %s\n" (String.escaped s); *)
   let v = Bencode.decode s in
 (*            lprintf "Decoded file: %s\n" (Bencode.print v);  *)
@@ -98,7 +98,7 @@ let decode_torrent s =
                     length_set := true
                 
                 | String key, _ -> 
-                    lprintf "other field [%s] in files\n" key
+                    if !verbose_msg_servers then lprintf "other field [%s] in files\n" key
                 | _ -> 
                     lprintf "other field in files\n"
             ) list;
@@ -135,12 +135,12 @@ let decode_torrent s =
                     | String "pieces", String pieces ->
                         file_pieces := pieces
                     | String key, _ -> 
-                        lprintf "other field [%s] in info\n" key
+                        if !verbose_msg_servers then lprintf "other field [%s] in info\n" key
                     | _ -> 
                         lprintf "other field in info\n"
                 ) list            
             | String key, _ -> 
-                lprintf "other field [%s] after info\n" key
+                if !verbose_msg_servers then lprintf "other field [%s] after info\n" key
             | _ -> 
                 lprintf "other field after info\n"
         ) list
