@@ -20,19 +20,20 @@
 open Printf2
 open BasicSocket
 open Md4
+open Options
   
 open CommonDownloads
-  
+open CommonSwarming  
 open CommonClient
 open CommonServer
 open CommonComplexOptions
 open CommonFile
-open Options
 open CommonTypes
-open DonkeyTypes
-open DonkeyOptions
 open CommonOptions
 open CommonGlobals
+  
+open DonkeyTypes
+open DonkeyOptions
 open DonkeyGlobals
 
 let shared_files_ini = create_options_file (
@@ -65,7 +66,7 @@ I know this is stupid, but "give the people what they want"..
 
 let create_online_sig () =
 
-	let most_users = ref 0 in
+  let most_users = ref Int64.zero in
 	let server_name= ref "" in
 	let server_ip = ref "" in
 	let server_port = ref 0 in
@@ -80,7 +81,7 @@ let create_online_sig () =
 
 	let oc = open_out (Filename.concat file_basedir "onlinesig.dat") in
 
-	if !most_users = 0 then
+  if !most_users = Int64.zero then
 		output_string oc ("0\n")
 	else 
 		output_string oc (Printf.sprintf "1|%s|%s|%d\n" !server_name !server_ip !server_port);
@@ -184,7 +185,7 @@ let client_to_value c =
 (*      "client_age", int_to_value (
         CommonGlobals.connection_last_conn 
         c.client_connection_control); *)
-      "client_last_filereqs", int_to_value c.client_last_filereqs;
+(*      "client_last_filereqs", int_to_value c.client_last_filereqs; *)
     ]
   in
   
@@ -201,33 +202,36 @@ let client_option =
   donkey_client_to_value 
 
 let value_to_server assocs = 
-      let get_value name conv = conv (List.assoc name assocs) in
-      let get_value_nil name conv = 
-        try conv (List.assoc name assocs) with _ -> []
-      in
-      let ip, port = get_value "server_addr" (fun v ->
-            match v with
-              List [ip;port] | SmallList [ip;port] ->
-                let ip = Ip.of_string (value_to_string ip) in
-                let port = value_to_int port in
-                ip, port
-            | _ -> failwith  "Options: Not an server option"
-        ) in
-      let l = DonkeyGlobals.new_server ip port !!initial_score in
-      
-      (try
-          l.server_description <- get_value "server_desc" value_to_string 
-        with _ -> ());
-      (try
-          l.server_name <- get_value "server_name" value_to_string
-        with _ -> ());
-      (try
-          connection_set_last_conn l.server_connection_control
-            (normalize_time (mini (get_value "server_age" value_to_int) 
-            (BasicSocket.last_time ())));
-        with _ -> ());
-      as_server l.server_server
-
+  let get_value name conv = conv (List.assoc name assocs) in
+  let get_value_nil name conv = 
+    try conv (List.assoc name assocs) with _ -> []
+  in
+  let ip, port = get_value "server_addr" (fun v ->
+        match v with
+          List [ip;port] | SmallList [ip;port] ->
+            let ip = Ip.of_string (value_to_string ip) in
+            let port = value_to_int port in
+            ip, port
+        | _ -> failwith  "Options: Not an server option"
+    ) in
+  let l = DonkeyGlobals.new_server ip port !!initial_score in
+  
+  (try
+      l.server_description <- get_value "server_desc" value_to_string 
+    with _ -> ());
+  (try
+      l.server_name <- get_value "server_name" value_to_string
+    with _ -> ());
+  (try
+      l.server_preferred <- get_value "server_preferred" value_to_bool
+    with _ -> ());
+  (try
+      connection_set_last_conn l.server_connection_control
+        (normalize_time (mini (get_value "server_age" value_to_int) 
+          (BasicSocket.last_time ())));
+    with _ -> ());
+  as_server l.server_server
+  
 let server_to_value c =
   let fields = 
     [
@@ -247,6 +251,10 @@ let server_to_value c =
       ("server_name", string_to_value c.server_name) :: fields
     else fields in
   
+  let fields = 
+    if c.server_preferred then
+      ("server_preferred", bool_to_value true) :: fields else
+      fields in
   List.rev fields
 
 let value_to_int32pair v =
@@ -311,14 +319,19 @@ let value_to_file file_size file_state assocs =
     with _ -> 
         lprintf "Could not load chunks states\n"; );
 *)
+  let md4s = get_value "file_md4s" (value_to_array value_to_md4) in
+  
+  if md4s <> [||] then file.file_computed_md4s <- md4s;
   
   (match file.file_swarmer with
       None -> ()
     | Some swarmer ->
         Int64Swarmer.value_to_swarmer swarmer assocs;
-        add_file_downloaded file.file_file
-          (Int64Swarmer.downloaded swarmer);
-
+        Int64Swarmer.set_verifier swarmer (if md4s = [||] then
+            VerificationNotAvailable
+          else
+            Verification
+            (Array.map (fun md4 -> Ed2k md4) md4s))
   );
   
 (*
@@ -342,8 +355,6 @@ let value_to_file file_size file_state assocs =
     else Array2.min file.file_chunks_age);
 *)
   
-  let md4s = get_value "file_md4s" (value_to_array value_to_md4) in
-  file.file_md4s <- (if md4s = [||] then file.file_md4s else md4s);
   file_md4s_to_register := file :: !file_md4s_to_register;
   as_file file
   
@@ -371,10 +382,10 @@ let file_to_value file =
         (List.map (fun (i1,i2) -> 
             SmallList [int64_to_value i1; int64_to_value i2])
         file.file_absent_chunks); *)
+      ("file_md4s", 
+        array_to_value Md4.hash_to_value file.file_computed_md4s);
       "file_filenames", List
         (List.map (fun (s,_) -> string_to_value s) file.file_filenames);
-      "file_md4s", array_to_value Md4.hash_to_value 
-        file.file_md4s;
 (*      "file_mtime", float_to_value (
         try Unix32.mtime (file_disk_name file) with _ -> 0.0) *)
     ]
@@ -382,8 +393,8 @@ let file_to_value file =
   let fields = 
   match file.file_swarmer with
     None -> fields
-  | Some swarmer ->
-      Int64Swarmer.swarmer_to_value swarmer fields
+    | Some swarmer ->
+        Int64Swarmer.swarmer_to_value swarmer fields
   in
   fields
     
@@ -552,22 +563,30 @@ let known_shared_files = define_option shared_section
     (list_option SharedFileOption.t) []
   
 (************  UPDATE OPTIONS *************)  
+
+(* This function is used only with the "n" command and the 
+op_network_add_server method, which are the only ways for the user
+to enter servers, bypassing the update_server_list variable. *)
   
-let add_server ip port =
+let force_add_server ip port =
   try
     DonkeyGlobals.find_server ip port
   with _ ->
       let s = DonkeyGlobals.new_server ip port !!initial_score in
       DonkeyGlobals.servers_ini_changed := true;
-      s
-  
+      s        
+      
 let check_add_server ip port =
-  if Ip.valid ip && not (is_black_address ip port) && port <> 4662 then
-    add_server ip port
+  if !!update_server_list &&
+    Ip.valid ip && Ip.reachable ip &&
+    not (is_black_address ip port) && port <> 4662 then
+    force_add_server ip port
   else raise Not_found
 
 let safe_add_server ip port =
-  if Ip.valid ip && not (is_black_address ip port) && port <> 4662 then
+  if !!update_server_list &&
+    Ip.valid ip && Ip.reachable ip &&
+    not (is_black_address ip port) && port <> 4662 then
     try
       ignore (DonkeyGlobals.find_server ip port)
     with _ ->
@@ -581,7 +600,7 @@ let remove_server ip port =
     DonkeyGlobals.remove_server ip port
   with _ -> ()
 
-  
+let config_files_loaded = ref false  
       
 let load _ =
   lprintf "LOADING SHARED FILES AND SOURCES\n"; 
@@ -590,7 +609,8 @@ let load _ =
       Options.load stats_ini;
       Options.load mod_stats_ini;
     with Sys_error _ ->
-        Options.save_with_help shared_files_ini)
+        Options.save_with_help shared_files_ini);
+  config_files_loaded := true
   
 let guptime = define_option stats_section ["guptime"] "" int_option 0
   
@@ -636,27 +656,37 @@ let _ =
 
 let diff_time = ref 0
 
+let sources_loaded = ref false  (* added 2.5.24 *)
+
 let save _ =
-  lprintf "SAVING SHARED FILES AND SOURCES\n";
-  Options.save_with_help shared_files_ini;
-  guptime =:= !!guptime + (last_time () - start_time) - !diff_time;
-  diff_time := (last_time () - start_time);
-  Options.save_with_help stats_ini;
-  Options.save_with_help mod_stats_ini;
-  
-  save_time =:= last_time ();
-  let cleaner = DonkeySources.attach_sources_to_file file_sources_section in
-  Options.save_with_help file_sources_ini;
-  cleaner ();
-  lprintf "SAVED\n"; 
-  create_online_sig ()
+  if !config_files_loaded then begin
+(*  lprintf "SAVING SHARED FILES AND SOURCES\n"; *)
+      Options.save_with_help shared_files_ini;
+      guptime =:= !!guptime + (last_time () - start_time) - !diff_time;
+      diff_time := (last_time () - start_time);
+      Options.save_with_help stats_ini;
+      Options.save_with_help mod_stats_ini;
+      create_online_sig ();
+    end;
+  if !sources_loaded then begin
+      save_time =:= last_time ();
+      let cleaner = DonkeySources.attach_sources_to_file file_sources_section in
+      Options.save_with_help file_sources_ini;
+      cleaner ()
+    end
+(*  lprintf "SAVED\n";  *)
     
 let guptime () = !!guptime - !diff_time
-
   
 let load_sources () = 
   (try 
+(*      lprintf "load_sources: loading sources\n"; *)
+      let cleaner = DonkeySources.attach_sources_to_file file_sources_section in
+      cleaner ();
       Options.load file_sources_ini;
+      cleaner ();
+      sources_loaded := true;
+(*      lprintf "load_sources: sources loaded\n"; *)
       List.iter (fun list ->
           let files = ref [] in
           List.iter (fun m ->
