@@ -151,11 +151,13 @@ let block_size = Int64.of_int 9728000
   
 let queue_timeout = ref (60. *. 10.) (* 10 minutes *)
     
-let files_queries_per_minute = 1
+(* ask connected servers for sources for 3 files
+   costs 3*16 credits; 1 minute gets us 60 credits *)
+let files_queries_per_minute = 3 (* costs 48 server-credits; we did get 60 (1 each second) *)
     
 let nclients = ref 0
 
-let protocol_version = 61
+let protocol_version = 62
 let max_file_groups = 1000
   
 let udp_sock = ref (None: UdpSocket.t option)
@@ -401,8 +403,13 @@ let remove_client_chunks file client_chunks =
   done
     *)
 
+let low_id ip =
+  match Ip.to_ints ip with
+    | _, _, _, 0 -> true
+    | _ -> false
+
 let is_black_address ip port =
-  !!black_list && (
+  !!black_list && not (low_id ip) && (
 (* lprintf "is black ="; *)
     not (ip_reachable ip) || (Ip.matches ip !!server_black_list) ||
     (List.mem port !!port_black_list) ||
@@ -437,7 +444,7 @@ let new_server ip port score =
         server_tags = [];
         server_nfiles = Int64.zero;
         server_nusers = Int64.zero;
-          server_max_users = 0;
+        server_max_users = 0;
         server_name = "";
         server_description = "";
         server_banner = "";
@@ -447,9 +454,10 @@ let new_server ip port score =
         server_last_message = 0;
         server_queries_credit = 0;
         server_waiting_queries = [];
+	server_sent_all_queries = false;
         server_id_requests = Fifo.create ();
-          server_flags = 0;
-          server_has_zlib = false;
+        server_flags = 0;
+        server_has_zlib = false;
       }
       and server_impl = 
         {
@@ -719,7 +727,22 @@ let remove_client c =
 let friend_remove c = 
   friend_remove  (as_client c)
 
-  
+(* Parts stolen from update_master_servers. Maybe someone competent
+   enough reduces the redundant code produced here. *)
+let last_connected_master () =
+  let server_list = connected_servers () in
+  let masters = ref [] in
+  List.iter (
+    fun s ->
+      if s.server_master then 
+        match s.server_sock with
+          | Connection _ -> 
+              masters := s :: !masters
+          | _ -> s.server_master <- false
+  ) server_list;
+  match !masters with
+  | s :: _ -> s
+  | [] -> raise Not_found
   
 let last_connected_server () =
   match !servers_list with 
@@ -871,7 +894,32 @@ let brand_mod_to_string b =
   | Brand_mod_bl4ckbird -> "bl4ckbird"
   | Brand_mod_bl4ckf0x -> "bl4ckf0x"
   | Brand_mod_rt -> "rt"
+  | Brand_mod_airionix -> "air-ionix"
+  | Brand_mod_ionix -> "ionix"
+  | Brand_mod_tornado -> "tornado"
+  | Brand_mod_antifaker -> "anti-faker"
+  | Brand_mod_netf -> "netf"
+  | Brand_mod_nextemf -> "nextemf"
+  | Brand_mod_proemule -> "proemule"
+  | Brand_mod_szemule -> "szemule"
+  | Brand_mod_darkmule -> "darkmule"
+  | Brand_mod_miragemod -> "miragemod"
+  | Brand_mod_nextevolution -> "nextevolution"
+  | Brand_mod_pootzgrila -> "pootzgrila"
+  | Brand_mod_freeangel -> "freeangel"
+  | Brand_mod_enos -> "enos"
+  | Brand_mod_webys -> "webys"
     
+let string_of_client c =
+  Printf.sprintf "client[%d] %s(%s) %s" (client_num c)
+  c.client_name (brand_to_string c.client_brand)
+  (match c.client_kind with
+      Indirect_address _ | Invalid_address _ -> ""
+    | Direct_address (ip,port) ->
+        Printf.sprintf  " [%s:%d]" (Ip.to_string ip) port;
+  )
+
+
 
 let check_result r tags =
   if r.result_names = [] || r.result_size = Int64.zero then begin
@@ -998,3 +1046,4 @@ let server_accept_multiple_getsources s =
 
 let server_send_multiple_replies s =
   (s.server_flags land DonkeyProtoUdp.PingServerReplyUdp.multiple_replies) <> 0
+
