@@ -110,7 +110,7 @@ let already_done = Failure "File already downloaded (use 'force_download' if nec
       
 let really_query_download filenames size md4 location old_file absents =
   
-  lprintf "really_query_download..................... \n";
+  if !verbose then lprintf "really_query_download..................... \n";
   begin
     try
       let file = Hashtbl.find files_by_md4 md4 in
@@ -174,8 +174,7 @@ let really_query_download filenames size md4 location old_file absents =
   DonkeyNeighbours.recover_downloads [file];
   
   List.iter (fun s ->
-      do_if_connected s.server_sock (fun sock ->
-          query_location file sock)
+    add_query_location file s
   ) (connected_servers());
 
   (try
@@ -439,7 +438,7 @@ let parse_donkey_url url =
   | "ed2k://" :: "server" :: ip :: port :: _
   | "server" :: ip :: port :: _ ->
       let ip = Ip.of_string ip in
-      let s = check_add_server ip (int_of_string port) in
+      let s = force_add_server ip (int_of_string port) in
       server_connect (as_server s.server_server);
       true 
   | "ed2k://" :: "friend" :: ip :: port :: _
@@ -531,7 +530,7 @@ let commands = [
         with e ->
             Printf.sprintf "error %s while loading temp files" (
               Printexc2.to_string e)
-    ), "<temp_dir> :\t\timport the old edonkey temp directory";
+    ), "<temp_dir> :\t\t\timport the old edonkey temp directory";
     
     "load_old_history", Arg_none (fun o ->
         let buf = o.conn_buf in
@@ -584,7 +583,7 @@ let commands = [
         brotherhood =:= 
           (List.map (fun file -> file.file_md4) !files) :: !!brotherhood;
         "    are now defined as colocated"
-    ) , "<f1> < f2> ... :\t\t\tdefine these files as probably colocated";
+    ) , "<f1> < f2> ... :\t\tdefine these files as probably colocated";
     
     "recover_bytes", Arg_multiple (fun args o ->
         let buf = o.conn_buf in
@@ -608,7 +607,7 @@ let commands = [
             ) !current_files
         ) args;
         ""
-    ) , "<f1> < f2> ... :\t\t\ttry to recover these files at byte level";
+    ) , "<f1> < f2> ... :\t\ttry to recover these files at byte level";
 
     "preferred", Arg_two (fun arg1 arg2 o ->
         let preferred = bool_of_string arg1 in
@@ -620,7 +619,7 @@ let commands = [
             end
         ) servers_by_key;
         "ok"
-    ), "<true/false> <ip> : set the server with this IP has preferred";
+    ), "<true/false> <ip> :\t\tset the server with this IP has preferred";
     
     "bs", Arg_multiple (fun args o ->
         List.iter (fun arg ->
@@ -642,8 +641,7 @@ let commands = [
           web_infos =:=  v :: !!web_infos;
         CommonWeb.load_url kind url;
         "url added to web_infos. downloading now"
-    ), "<kind> <url> :\t\t\tload this file from the web.
-\t\t\t\t\tkind is either server.met (if the downloaded file is a server.met)";
+    ), "<kind> <url> :\t\t\tload this file from the web. Kind is either server.met (if the downloaded file is a server.met)";
     
     "scan_temp", Arg_none (fun o ->
         let buf = o.conn_buf in
@@ -738,6 +736,12 @@ parent.fstatus.location.href='submit?q=rename+'+i+'+\\\"'+renameTextOut+'\\\"';
         "done"
     ), ":\t\t\t\tshow sources currently known";
 
+    "tsources", Arg_none (fun o ->
+        let buf = o.conn_buf in
+        DonkeySources.print_tsources buf;
+        "done"
+    ), ":\t\t\t\tshow table of sources of downloading files";
+
     (*
     "update_sources", Arg_none (fun o ->
         let buf = o.conn_buf in
@@ -789,7 +793,7 @@ parent.fstatus.location.href='submit?q=rename+'+i+'+\\\"'+renameTextOut+'\\\"';
             port_black_list =:=  port :: !!port_black_list;
         ) args;
         "done"
-    ), "<port1> <port2> ... :\t\tadd these Ports to the port black list";
+    ), "<port1> <port2> ... :\t\t\tadd these Ports to the port black list";
     
     "send_servers", Arg_none (fun o ->
         CommonWeb.connect_redirector ();
@@ -809,47 +813,13 @@ DonkeySources.recompute_ready_sources ()        *)
       ()
   );
   file_ops.op_file_pause <- (fun file -> ()  );
-  file_ops.op_file_print_sources_html_header <- (fun file buf info ->
-      
-      html_mods_table_header buf "sourcesTable" "sources al" ([ 
-          ( "1", "srh ac", "Client number (click to add as friend)", "Num" ) ; 
-          ( "0", "srh", "[A] = Active download from client", "A" ) ; 
-          ( "0", "srh", "Client state", "CS" ) ; 
-          ( "0", "srh", "Client name", "Name" ) ; 
-          ( "0", "srh", "Client brand", "CB" ) ; 
-        ] @
-          (if !!emule_mods_count then [( "0", "srh", "eMule MOD", "EM" )] else [])
-        @ [
-          ( "0", "srh", "Overnet [T]rue, [F]alse", "O" ) ; 
-          ( "0", "srh", "Connection [I]ndirect, [D]irect", "C" ) ; 
-          ( "0", "srh br", "IP address", "IP address" ) ; 
-          ( "1", "srh ar", "Total UL bytes to this client for all files", "UL" ) ; 
-          ( "1", "srh ar br", "Total DL bytes from this client for all files", "DL" ) ; 
-          ( "1", "srh ar", "Your queue rank on this client", "Rnk" ) ; 
-          ( "1", "srh ar br", "Source score", "Scr" ) ; 
-          ( "1", "srh ar", "Last ok (minutes)", "LO" ) ; 
-          ( "1", "srh ar", "Last try (minutes)", "LT" ) ; 
-          ( "1", "srh ar br", "Next try (minutes)", "NT" ) ; 
-          ( "0", "srh", "Has a slot [T]rue, [F]alse", "H" ) ; 
-          ( "0", "srh br", "Banned [T]rue, [F]alse", "B" ) ; 
-          ( "1", "srh ar", "Requests sent", "RS" ) ; 
-          ( "1", "srh ar", "Requests received", "RR" ) ; 
-          ( "1", "srh ar br", "Connected time (minutes)", "CT" ) ; 
-          ( "0", "srh br", "Client MD4", "MD4" ) ; 
-          ( "0", "srh", "Chunks (absent|partial|present|verified)", 
-            (colored_chunks 
-                (Array.init (String.length info.G.file_chunks)
-                (fun i -> ((int_of_char info.G.file_chunks.[i])-48)))) ) ; 
-          ( "1", "srh ar", "Number of full chunks", (Printf.sprintf "%d"
-                (let fc = ref 0 in (String.iter (fun s -> if s = '2' then incr fc) 
-                  info.G.file_chunks );!fc ))) ])
-  );
   
   file_ops.op_file_commit <- (fun file new_name ->
       
 (*      DonkeyStats.save_download_history file; *)
       
-      if not (List.mem file.file_md4 !!old_files) then
+      if !!keep_cancelled_in_old_files &&
+        not (List.mem file.file_md4 !!old_files) then
         old_files =:= file.file_md4 :: !!old_files;
       DonkeyShare.remember_shared_info file new_name;
       
@@ -1084,10 +1054,205 @@ let _ =
           list := (as_client c) :: !list
       ) file.file_sources;
       !list
+  );
+  file_ops.op_file_print_html <- (fun file buf ->
+
+     html_mods_cntr_init ();
+
+     let tr () = 
+      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ())
+     in 
+
+      tr ();
+      html_mods_td buf [
+        ("Fake check links", "sr br", "Fakecheck");
+        ("", "sr", Printf.sprintf "\\<a target=\\\"_blank\\\" href=\\\"http://donkeyfakes.gambri.net/fakecheck.php\\?hash=%s\\\"\\>[Donkey-Fakes]\\</a\\>"
+            (Md4.to_string file.file_md4)
+        ) ];
+
+      tr ();
+      html_mods_td buf [
+        ("File History Links", "sr br", "File History");
+	("","sr", Printf.sprintf "\\<a target=\\\"_blank\\\" href=\\\"http://stats.razorback2.com/ed2khistory\\?ed2k=%s\\\"\\>RazorBack File History\\</a\\>"
+	    (Md4.to_string file.file_md4)
+	)
+      ];
+      tr ();
+      html_mods_td buf [
+        ("ed2k link", "sr br", "ed2k link");
+        ("", "sr", Printf.sprintf "\\<a href=\\\"ed2k://|file|%s|%s|%s|/\\\"\\>ed2k://|file|%s|%s|%s|/\\</A\\>"
+            (file_best_name file)
+          (Int64.to_string (file_size file))
+          (Md4.to_string file.file_md4)
+          (file_best_name file)
+          (Int64.to_string (file_size file))
+          (Md4.to_string file.file_md4)) ];
+
+      tr ();
+      let optionlist = ref "" in
+      List.iter (fun (name,_) ->
+          optionlist := !optionlist ^ Printf.sprintf "\\<option value=\\\"%s\\\"\\>%s\n" name name;
+      ) file.file_filenames;
+
+
+      let input_size = (Printf.sprintf "%d" ((String.length (file_best_name file))+3)) in
+      html_mods_td buf [
+        ("Rename file by selecting an alternate name", "sr br", "Filename");
+        ("Rename file", "sr",
+          Printf.sprintf "\\<script language=javascript\\>
+\\<!-- 
+function submitRenameForm(i) {
+var formID = document.getElementById(\\\"renameForm\\\" + i)
+var regExp = new RegExp (' ', 'gi') ;
+var renameTextOut = formID.newName.value.replace(regExp, '+'); 
+parent.fstatus.location.href='submit?q=rename+%d+\\\"'+renameTextOut+'\\\"';
+}
+//--\\>
+\\</script\\>" (file_num file)
+
+   ^ "\\<table border=0 cellspacing=0 cellpadding=0\\>\\<tr\\>\\<td\\>"
+   ^ "\\<form name=\\\"renameForm1\\\" id=\\\"renameForm1\\\" action=\\\"javascript:submitRenameForm(1);\\\"\\>"
+   ^ "\\<select name=\\\"newName\\\" id=\\\"newName\\\" onchange=\\\"this.form.submit()\\\"\\>"
+   ^ Printf.sprintf "\\<option value=\\\"%s\\\" selected\\>%s\n" (file_best_name file) (file_best_name file)
+   ^ !optionlist
+   ^ "\\</select\\>\\</td\\>\\</tr\\>\\</form\\>\\<tr\\>\\<td\\>\\<form name=\\\"renameForm2\\\" id=\\\"renameForm2\\\" action=\\\"javascript:submitRenameForm(2);\\\"\\>"
+   ^ "\\<input name=\\\"newName\\\" type=text size=" ^ input_size ^ " value=\\\"" ^ (file_best_name file) ^ "\\\"\\>\\</input\\>\\</td\\>\\</tr\\>\\</form\\>\\</table\\>"
+    ) ];
+
+
   );
   file_ops.op_file_print_sources_html <- (fun file buf ->
-(* TODO:       if !!source_management = 3 then DonkeySources.print_sources_html file buf *)
-      ()
+
+   if List.length (file_ops.op_file_all_sources file) > 0 then begin
+
+      let chunks = 
+      (match file.file_swarmer with
+       None -> "" | Some swarmer ->
+       Int64Swarmer.verified_bitmap swarmer)
+      in
+
+
+      html_mods_table_header buf "sourcesTable" "sources al" ([ 
+          ( "1", "srh ac", "Client number (click to add as friend)", "Num" ) ; 
+          ( "0", "srh", "[A] = Active download from client", "A" ) ; 
+          ( "0", "srh", "Client state", "CS" ) ; 
+          ( "0", "srh", "Client name", "Name" ) ; 
+          ( "0", "srh", "Client brand", "CB" ) ; 
+        ] @
+          (if !!emule_mods_count then [( "0", "srh", "eMule MOD", "EM" )] else [])
+        @ [
+          ( "0", "srh", "Overnet [T]rue, [F]alse", "O" ) ; 
+          ( "0", "srh", "Connection [I]ndirect, [D]irect", "C" ) ; 
+          ( "0", "srh br", "IP address", "IP address" ) ; 
+          ( "1", "srh ar", "Total UL bytes to this client for all files", "UL" ) ; 
+          ( "1", "srh ar br", "Total DL bytes from this client for all files", "DL" ) ; 
+          ( "1", "srh ar", "Your queue rank on this client", "Rnk" ) ; 
+          ( "1", "srh ar br", "Source score", "Scr" ) ; 
+          ( "1", "srh ar", "Last ok (minutes)", "LO" ) ; 
+          ( "1", "srh ar", "Last try (minutes)", "LT" ) ; 
+          ( "1", "srh ar br", "Next try (minutes)", "NT" ) ; 
+          ( "0", "srh", "Has a slot [T]rue, [F]alse", "H" ) ; 
+          ( "0", "srh br", "Banned [T]rue, [F]alse", "B" ) ; 
+          ( "1", "srh ar", "Requests sent", "RS" ) ; 
+          ( "1", "srh ar", "Requests received", "RR" ) ; 
+          ( "1", "srh ar br", "Connected time (minutes)", "CT" ) ; 
+          ( "0", "srh br", "Client MD4", "MD4" ) ; 
+          ( "0", "srh", "Chunks (absent|partial|present|verified)", 
+            (colored_chunks 
+                (Array.init (String.length chunks)
+                (fun i -> ((int_of_char chunks.[i])-48)))) ) ; 
+          ( "1", "srh ar", "Number of full chunks", (Printf.sprintf "%d"
+                (let fc = ref 0 in (String.iter (fun s -> if s = '2' then incr fc) 
+                  chunks );!fc ))) ]);
+
+
+      let counter = ref 0 in
+      DonkeySources.iter_all_sources (fun s -> 
+          let s_uid = s.DonkeySources.source_uid in
+          let c = new_client s_uid in
+          let c1 = (as_client c) in
+          incr counter;
+
+        try 
+          let i = (client_info (as_client c)) in
+
+
+          Printf.bprintf buf "\\<tr onMouseOver=\\\"mOvr(this);\\\" 
+        onMouseOut=\\\"mOut(this);\\\" class=\\\"%s\\\"\\>"
+           (if (!counter mod 2 == 0) then "dl-1" else "dl-2");
+
+          
+          Printf.bprintf buf "\\<td title=\\\"Add as Friend\\\" class=\\\"srb ar\\\"
+    onClick=\\\"parent.fstatus.location.href='submit?q=friend_add+%d'\\\"\\>%d\\</TD\\>"
+            (client_num c) (client_num c);
+          
+          html_mods_td buf ([
+            ("", "sr", (match c.client_download with 
+                  None -> Printf.sprintf "" 
+                | Some _ -> Printf.sprintf "%s" ( 
+                      let qfiles = c.client_file_queue in
+                      let (qfile, qchunks,_) =  List.hd qfiles in
+                      if (qfile == file) then
+                        "A" else "";)) );
+            ((string_of_connection_state (client_state c)), "sr", 
+              (short_string_of_connection_state (client_state c)) );
+            (String.escaped c.client_name, "sr", client_short_name c.client_name);
+            (brand_to_string c.client_brand, "sr", gbrand_to_string c.client_brand);
+            ] @
+            (if !!emule_mods_count then [(brand_mod_to_string c.client_mod_brand, "sr", gbrand_mod_to_string c.client_mod_brand)] else [])
+            @ [
+              ("", "sr", (if DonkeySources.source_brand c.client_source
+                  then "T" else "F"));
+            ("", "sr", (match c.client_kind with 
+                  | Direct_address (ip,port) -> Printf.sprintf "D"
+                  | _ -> Printf.sprintf "I"
+                ));
+            ("", "sr br", match c.client_kind with
+                Direct_address (ip,port) -> Printf.sprintf "%s" (Ip.to_string ip)
+              | _ -> (string_of_client_addr c));
+            ("", "sr ar", (size_of_int64 c.client_uploaded));
+            ("", "sr ar br", (size_of_int64 c.client_downloaded));
+            ("", "sr ar", Printf.sprintf "%d" c.client_rank);
+            ("", "sr ar br", Printf.sprintf "%d" c.client_source.DonkeySources.source_score);
+            ("", "sr ar", (string_of_date (c.client_source.DonkeySources.source_age)));
+            ("", "sr ar", ("-"));
+            ("", "sr ar br", "-");
+            ("", "sr ar", (if client_has_a_slot (as_client c) then "T" else "F"));
+            ("", "sr ar br", (if c.client_banned then "T" else "F"));
+            ("", "sr ar", Printf.sprintf "%d" c.client_requests_sent);
+            ("", "sr ar", Printf.sprintf "%d" c.client_requests_received);
+            ("", "sr ar br", Printf.sprintf "%d" (((last_time ()) - c.client_connect_time) / 60));
+            ("", "sr br", (Md4.to_string c.client_md4)); ]);
+          
+          Printf.bprintf buf "\\<td class=\\\"sr \\\"\\>";
+          
+          ( let qfiles = c.client_file_queue in
+            if qfiles <> [] then begin
+                try
+                  let _, qchunks,_ = List.find (fun (qfile, _,_) ->
+                        qfile == file) qfiles in
+                  let tc = ref 0 in
+                  Printf.bprintf buf "%s\\</td\\>\\<td class=\\\"sr ar\\\"\\>%d\\</td\\>" 
+                    (CommonFile.colored_chunks (Array.init (Array.length qchunks) 
+                      (fun i -> (if qchunks.(i) then 2 else 0)) )) 
+                  (Array.iter (fun b -> if b then incr tc) qchunks;!tc);
+                with Not_found -> (
+                      Printf.bprintf buf "\\</td\\>\\<td class=\\\"sr ar\\\"\\>\\</td\\>" 
+                    );
+              end
+            else
+              Printf.bprintf buf "\\</td\\>\\<td class=\\\"sr ar\\\"\\>\\</td\\>" 
+          );
+
+           Printf.bprintf buf "\\</tr\\>";
+        with _ -> ()
+
+      ) file.file_sources;
+
+    Printf.bprintf buf "\\</table\\>\\</div\\>\\<br\\>";
+
+   end;
+
   );
   file_ops.op_file_cancel <- (fun file ->
       Hashtbl.remove files_by_md4 file.file_md4;
@@ -1213,79 +1378,6 @@ lprint_newline ();
         c.client_name
         (string_of_date (c.client_source.DonkeySources.source_age))
   );
-  
-  
-  client_ops.op_client_bprint_html <- (fun c buf file ->
-      
-      begin
-        try 
-          let i = (client_info (as_client c)) in
-          
-          Printf.bprintf buf "\\<td title=\\\"Add as Friend\\\" class=\\\"srb ar\\\"
-		onClick=\\\"parent.fstatus.location.href='submit?q=friend_add+%d'\\\"\\>%d\\</TD\\>"
-            (client_num c) (client_num c);
-          
-          html_mods_td buf ([
-            ("", "sr", (match c.client_download with 
-                  None -> Printf.sprintf "" 
-                | Some _ -> Printf.sprintf "%s" ( 
-                      let qfiles = c.client_file_queue in
-                      let (qfile, qchunks,_) =  List.hd qfiles in
-                      if (qfile == (as_file_impl file).impl_file_val) then
-                        "A" else "";)) );
-            ((string_of_connection_state (client_state c)), "sr", 
-              (short_string_of_connection_state (client_state c)) );
-            (String.escaped c.client_name, "sr", client_short_name c.client_name);
-            (brand_to_string c.client_brand, "sr", gbrand_to_string c.client_brand);
-            ] @
-            (if !!emule_mods_count then [(brand_mod_to_string c.client_mod_brand, "sr", gbrand_mod_to_string c.client_mod_brand)] else [])
-            @ [
-              ("", "sr", (if DonkeySources.source_brand c.client_source
-                  then "T" else "F"));
-            ("", "sr", (match c.client_kind with 
-                  | Direct_address (ip,port) -> Printf.sprintf "D"
-                  | _ -> Printf.sprintf "I"
-                ));
-            ("", "sr br", match c.client_kind with
-                Direct_address (ip,port) -> Printf.sprintf "%s" (Ip.to_string ip)
-              | _ -> (string_of_client_addr c));
-            ("", "sr ar", (size_of_int64 c.client_uploaded));
-            ("", "sr ar br", (size_of_int64 c.client_downloaded));
-            ("", "sr ar", Printf.sprintf "%d" c.client_rank);
-            ("", "sr ar br", Printf.sprintf "%d" c.client_score);
-            ("", "sr ar", (string_of_date (c.client_source.DonkeySources.source_age)));
-            ("", "sr ar", ("-"));
-            ("", "sr ar br", "-");
-            ("", "sr ar", (if client_has_a_slot (as_client c) then "T" else "F"));
-            ("", "sr ar br", (if c.client_banned then "T" else "F"));
-            ("", "sr ar", Printf.sprintf "%d" c.client_requests_sent);
-            ("", "sr ar", Printf.sprintf "%d" c.client_requests_received);
-            ("", "sr ar br", Printf.sprintf "%d" (((last_time ()) - c.client_connect_time) / 60));
-            ("", "sr br", (Md4.to_string c.client_md4)); ]);
-          
-          Printf.bprintf buf "\\<td class=\\\"sr \\\"\\>";
-          
-          ( let qfiles = c.client_file_queue in
-            if qfiles <> [] then begin
-                try
-                  let _, qchunks,_ = List.find (fun (qfile, _,_) ->
-                        qfile == (as_file_impl file).impl_file_val) qfiles in
-                  let tc = ref 0 in
-                  Printf.bprintf buf "%s\\</td\\>\\<td class=\\\"sr ar\\\"\\>%d\\</td\\>" 
-                    (CommonFile.colored_chunks (Array.init (Array.length qchunks) 
-                      (fun i -> (if qchunks.(i) then 2 else 0)) )) 
-                  (Array.iter (fun b -> if b then incr tc) qchunks;!tc);
-                with Not_found -> (
-                      Printf.bprintf buf "\\</td\\>\\<td class=\\\"sr ar\\\"\\>\\</td\\>" 
-                    );
-              end
-            else
-              Printf.bprintf buf "\\</td\\>\\<td class=\\\"sr ar\\\"\\>\\</td\\>" 
-          );
-        with _ -> ()
-      end;
-  );
-  
   client_ops.op_client_dprint <- (fun c o file ->
       let info = file_info file in
       let buf = o.conn_buf in
@@ -1377,7 +1469,7 @@ let _ =
   
 let _ =
   shared_ops.op_shared_unshare <- (fun file ->
-      lprintf "************ UNSHARE FILE ************\n";
+      if !verbose_share then lprintf "************ UNSHARE FILE ************\n";
       unshare_file file;
       
 (* Should we or not ??? *)
