@@ -131,9 +131,9 @@ let declare_completed_file file =
   file_completed (as_file file);
   
   unshare_file file;
-  ignore (CommonShared.new_shared "completed" 0 (
+  (*ignore (CommonShared.new_shared "completed" 0 (
       file_best_name file )
-    (file_disk_name file));
+    (file_disk_name file));*)
   (try
       let format = CommonMultimedia.get_info
           (file_disk_name file) in
