@@ -35,8 +35,10 @@ open DonkeyOptions
 open CommonOptions
 open DonkeyComplexOptions
 open DonkeyGlobals
-  
-let must_share_file file codedname has_old_impl =
+
+let new_shared_files = ref []
+
+let must_share_file shdir file codedname has_old_impl =
   match file.file_shared with
   | Some _ -> ()
   | None ->
@@ -45,6 +47,7 @@ let must_share_file file codedname has_old_impl =
           impl_shared_update = 1;
           impl_shared_fullname = file_disk_name file;
           impl_shared_codedname = codedname;
+          impl_shared_directory = shdir;
           impl_shared_size = file_size file;
           impl_shared_id = file.file_md4;
           impl_shared_num = 0;
@@ -54,6 +57,7 @@ let must_share_file file codedname has_old_impl =
           impl_shared_requests = 0;
         } in
       file.file_shared <- Some impl;
+      new_shared_files := file :: !new_shared_files;
       incr CommonGlobals.nshared_files;
       CommonShared.shared_calculate_total_bytes ();
       match has_old_impl with
@@ -70,7 +74,7 @@ let md4_of_array md4s =
   done;
   Md4.string s
   
-let new_file_to_share sh codedname old_impl =
+let new_file_to_share shdir sh codedname old_impl =
   try
 (* How do we compute the total MD4 of the file ? *)
     
@@ -86,7 +90,7 @@ let new_file_to_share sh codedname old_impl =
     end;
     
     let file = new_file FileShared sh.sh_name md4 sh.sh_size false in
-    must_share_file file codedname old_impl;
+    must_share_file shdir file codedname old_impl;
     file.file_computed_md4s <- md4s;
     file_md4s_to_register := file :: !file_md4s_to_register;
     let sh_name = Filename.basename sh.sh_name in
@@ -122,7 +126,7 @@ let new_file_to_share sh codedname old_impl =
 		 *)
                 lprintf "verified map of %s = %s\n"
 		         (codedname) (Int64Swarmer.verified_bitmap s))
-      | None -> lprintf "no swarmer for %s\n" codedname;
+      | None -> if !verbose_share then lprintf "no swarmer for %s\n" codedname;
     (try 
         file.file_format <- CommonMultimedia.get_info 
           (file_disk_name file)
@@ -149,24 +153,31 @@ let all_shared () =
         None -> ()
       | Some _ ->  shared_files := file :: !shared_files
   ) files_by_md4;
-  lprintf "%d files shared\n" (List.length !shared_files);
+  if !verbose_share then lprintf "%d files shared\n" (List.length !shared_files);
   !shared_files
 
+
 (*  Check whether new files are shared, and send them to connected servers.
 Do it only once per 5 minutes to prevent sending to many times all files.
   Should I only do it for master servers, no ?
+  
+  FritzMock: 21.01.2005 just send *new* shared files to servers, they never forget a clients files untill disconnection
   *)
 let send_new_shared () =
-  if !new_shared then
-    begin
-      new_shared := false;
-      let socks = ref [] in
-      let list = all_shared () in
-      List.iter (fun s ->
-          if s.server_master then
-            do_if_connected s.server_sock (fun sock ->
-                server_send_share s.server_has_zlib sock list))
-      (connected_servers ());
+	if !new_shared then begin
+		new_shared := false;
+		let socks = ref [] in
+		if !new_shared_files <> [] then begin
+			List.iter (fun s ->
+				if s.server_master then
+					do_if_connected s.server_sock (fun sock ->
+						server_send_share s.server_has_zlib sock !new_shared_files))
+			(connected_servers ());
+			lprintf "donkeyShare: send_new_share: Sent %d new shared files to servers\n" (List.length !new_shared_files);
+			new_shared_files := []
+			end
+		else
+			lprintf "donkeyShare: send_new_share: No new shared files to send to servers\n"
     end
           
 (*
@@ -228,7 +239,11 @@ let check_shared_files () =
                     Hashtbl.add shared_files_info 
                       (s.sh_name, s.sh_size, s.sh_mtime) s;
                     known_shared_files =:= s :: !!known_shared_files;
-                    new_file_to_share s sh.shared_shared.impl_shared_codedname (Some  sh.shared_shared);
+                    new_file_to_share 
+		      sh.shared_shared.impl_shared_directory 
+		      s 
+		      sh.shared_shared.impl_shared_codedname 
+		      (Some  sh.shared_shared);
                     shared_remove  sh.shared_shared;
                   end
                 else
@@ -242,9 +257,9 @@ let check_shared_files () =
 let local_dirname = Sys.getcwd ()
   
 let _ =
-  network.op_network_share <- (fun fullname codedname size ->
-      lprintf "********** NETWORK SHARE ************\n";
+  network.op_network_share <- (fun shdir fullname codedname size ->
       if !verbose_share then begin
+	  lprintf "********** NETWORK SHARE ************\n";
           lprintf "FULLNAME %s\n" fullname; 
         end;
 (*      let codedname = Filename.basename codedname in*)
@@ -262,7 +277,7 @@ lprintf "Searching %s" fullname; lprint_newline ();
             if !verbose_share then begin 
                 lprintf "USING OLD MD4s for %s\n" fullname;
               end;
-            new_file_to_share s codedname None
+            new_file_to_share shdir s codedname None
 (*          end else begin
             if !verbose_share then begin                
                 lprintf "Shared file %s has been modified\n" fullname;
@@ -289,6 +304,7 @@ lprintf "Searching %s" fullname; lprint_newline ();
               impl_shared_update = 1;
               impl_shared_fullname = fullname;
               impl_shared_codedname = codedname;
+              impl_shared_directory = shdir;
               impl_shared_size = size;
               impl_shared_num = 0;
               impl_shared_uploaded = Int64.zero;
@@ -345,4 +361,4 @@ let remember_shared_info file new_name =
         lprintf "Exception %s in remember_shared_info\n"
           (Printexc2.to_string e)
         
-let must_share_file file = must_share_file file (file_best_name file) None
+let must_share_file file =  must_share_file (share_to_commit (as_file file)) file (file_best_name file) None
