@@ -129,9 +129,6 @@ let value_to_file file_size file_state assocs =
       None -> ()
     | Some swarmer ->
         Int64Swarmer.value_to_swarmer swarmer assocs;
-        add_file_downloaded file.file_file
-          (Int64Swarmer.downloaded swarmer);
-        
   );
 (*
   (try 
@@ -170,7 +167,7 @@ let file_to_value file =
       "file_downloaded", int64_to_value (file_downloaded file);
       "file_id", string_to_value (Md4.to_string file.file_id);
       "file_sources", 
-      list_to_value "Gnutella Sources" (fun c ->
+      list_to_value (fun c ->
           match (find_download file c.client_downloads).download_uri with
             FileByIndex (i,n) -> 
               SmallList [ClientOption.client_to_value c; int_to_value i; 
@@ -180,7 +177,7 @@ let file_to_value file =
                 string_to_value s] 
       ) file.file_clients
       ;
-      "file_uids", list_to_value "toto" (fun uid ->
+      "file_uids", list_to_value (fun uid ->
           string_to_value  (Uid.to_string uid))
       file.file_uids;
 (*      "file_present_chunks", List
@@ -206,7 +203,7 @@ let save_config () =
   
   Queue.iter (fun (_,h) -> 
       try
-        let o = match h.host_kind with true -> ultrapeers | _ -> peers in
+        let o = match h.host_kind with Ultrapeer -> ultrapeers | _ -> peers in
 
 (* Don't save hosts that are older than 1 hour, and not responding *)
         if max h.host_connected h.host_age > last_time () - 3600 then
