@@ -149,13 +149,6 @@ let results_by_uid = Hashtbl.create 127
 
 ****************************************************************)
   
-  
-  
-(* Hosts are first injected in workflow. The workflow ensures that any
-host object is inspected every two minutes. *)
-let (workflow : host Queue.t) = 
-  Queues.workflow (fun time -> time + 120 > last_time ())
-
 let ready _ = false
   
 (* From the main workflow, hosts are moved to these workflows when they
@@ -186,7 +179,7 @@ module H = CommonHosts.Make(struct
           Tcp_Connect, 
           (600, (fun kind ->
                 [ match kind with
-                  | true -> ultrapeers_waiting_queue
+                  | Ultrapeer -> ultrapeers_waiting_queue
                   | (_) -> peers_waiting_queue
                 ]
             ));
@@ -196,54 +189,13 @@ module H = CommonHosts.Make(struct
             ))]
 
       let default_requests kind = [Tcp_Connect,0; Udp_Connect,0]
+        
+      let max_ultrapeers = max_known_ultrapeers
+      let max_peers = max_known_peers
     end)
-  
-(*
-let host_queue_add q h time =
-  if not (List.memq q h.host_queues) then begin
-      Queue.put q (time, h);
-      h.host_queues <- q :: h.host_queues
-    end
-
-let host_queue_take q =
-  let (time,h) = Queue.take q in
-  if List.memq q h.host_queues then begin
-      h.host_queues <- List2.removeq q h.host_queues 
-    end;
-  h
-
-let hosts_counter = ref 0
-  
-let new_host ip port ultrapeer kind = 
-  let key = (ip,port) in
-  try
-    let h = Hashtbl.find hosts_by_key key in
-    h.host_age <- last_time ();
-    h
-  with _ ->
-      incr hosts_counter;
-      let host = {
-          host_num = !hosts_counter;
-          host_server = None;
-          host_ip = ip;
-          host_port = port;
-          
-          host_age = last_time ();
-          host_tcp_request = 0;
-          host_udp_request = 0;
-          host_connected = 0;
-          
-          host_kind = kind;
-          host_ultrapeer = ultrapeer;
-          host_queues = [];
-        } in
-      Hashtbl.add hosts_by_key key host;
-      host_queue_add workflow host 0;
-      host
-        *)
 
 let new_server ip port =
-  let h = H.new_host ip port true in
+  let h = H.new_host ip port Ultrapeer in
   match h.host_server with
     Some s -> s
   | None ->
@@ -252,17 +204,16 @@ let new_server ip port =
           server_host = h;
           server_sock = NoConnection;
           server_agent = "<unknown>";
-          server_nfiles = 0;
+          server_nfiles = Int64.zero;
           server_nkb = 0;
-          server_nusers = 0;
+          server_nusers = Int64.zero;
           server_need_qrt = true;
           server_ping_last = Md4.random ();
-          server_nfiles_last = 0;
+          server_nfiles_last = Int64.zero;
           server_nkb_last = 0;
           server_vendor = "";
           
           server_connected = zero;
-(*          server_gnutella2 = false; *)
           server_query_key = NoUdpSupport;
         } and
         server_impl = {
@@ -273,7 +224,7 @@ let new_server ip port =
       server_add server_impl;
       h.host_server <- Some s;
       s
-
+      
 let extract_uids arg = Uid.expand [Uid.of_string arg]
   
 let result_sources = Hashtbl.create 1000
@@ -385,16 +336,17 @@ let new_file file_id file_name file_size =
   in
   Hashtbl.add searches_by_uid search.search_uid search;
   lprintf "SET SIZE : %Ld\n" file_size;
-  let swarmer = Int64Swarmer.create (as_file file) megabyte 
-    (Int64.of_int (256 * 1024)) 
-  in
+  let kernel = Int64Swarmer.create_swarmer file_temp file_size 
+      (Int64.of_int (256 * 1024))  in
+  let swarmer = Int64Swarmer.create kernel (as_file file) megabyte in
   file.file_swarmer <- Some swarmer;
+  (*
   Int64Swarmer.set_writer swarmer (fun offset s pos len ->      
       if !!CommonOptions.buffer_writes then 
         Unix32.buffered_write_copy t offset s pos len
       else
         Unix32.write  t offset s pos len
-  );
+  ); *)
   current_files := file :: !current_files;
   file_add file_impl FileDownloading;
 (*      lprintf "ADD FILE TO DOWNLOAD LIST\n"; *)
