@@ -17,10 +17,14 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
 
+open Unix
 open Printf2
 open Options
 
 let debug = ref false
+let loop_delay = ref 0.02
+let socketKeepalive = ref false
+let verbose_bandwidth = ref 0
 
 (* Try to better display the errors related to connections *)
 
@@ -255,6 +259,7 @@ let create_blocking name fd handler =
   
 let create name fd handler =
   MlUnix.set_nonblock fd;
+  if not Autoconf.windows then setsockopt fd SO_KEEPALIVE !socketKeepalive;
   create_blocking name fd handler
   
 external select: t list -> float -> unit = "ml_select" 
@@ -381,9 +386,6 @@ let rec exec_timers = function
       );
       exec_timers tail
       
-let loop_delay = ref 0.02
-
-let verbose_bandwidth = ref 0
   
 let bandwidth_second_timers = ref []
 let add_bandwidth_second_timer f =
