@@ -285,15 +285,15 @@ headers;
               with _ -> 
                   lprintf "bad content length [%s]\n" content;
         ) headers;
-        
-        let content_handler = content_handler !content_length headers in
+        let location = "Location", Url.to_string old_url in
+        let content_handler = content_handler !content_length (location::headers) in
         set_reader sock content_handler;
         let buf = TcpBufferedSocket.buf sock in
         if buf.len > 0 then
           content_handler sock buf.len 
     
-    | 302 ->
-        lprintf "Http_client 302: Redirect\n";
+    | 301 | 302 | 304 ->
+        lprintf "Http_client %d: Redirect\n" ans_code;
         if level < 10 then
           begin
             try
